@@ -1,6 +1,7 @@
 /*
  * authusekey - decode a key from ascii and use it
  */
+#include <config.h>
 #include <stdio.h>
 #include <ctype.h>
 
@@ -22,14 +23,12 @@ authusekey(
 	const u_char *str
 	)
 {
-	const u_char *cp;
-	int len;
+	size_t	len;
 
-	cp = str;
-	len = strlen((const char *)cp);
-	if (len == 0)
+	len = strlen((const char *)str);
+	if (0 == len)
 		return 0;
 
-	MD5auth_setkey(keyno, keytype, str, (int)strlen((const char *)str));
+	MD5auth_setkey(keyno, keytype, str, len);
 	return 1;
 }
