@@ -9,14 +9,19 @@
  * (Some code shamelessly based on the original NTP discrete event simulator)
  */
 
+#include <config.h>
 #ifdef SIM
 #include "ntpd.h"
-#include "ntpsim.h"
-#include "ntp_data_structures.h"
+#include "ntp_config.h"
 
+/* forward prototypes */
+int determine_event_ordering(const Event *e1, const Event *e2);
+int determine_recv_buf_ordering(const struct recvbuf *b1, 
+				const struct recvbuf *b2);
+void create_server_associations(void);
+void init_sim_io(void);
 
 /* Global Variable Definitions */
-
 sim_info simulation;		/* Simulation Control Variables */
 local_clock_info simclock;	/* Local Clock Variables */
 queue *event_queue;		/* Event Queue */
@@ -28,133 +33,175 @@ void (*event_ptr[]) (Event *) = {
 };			/* Function pointer to the events */
 
 
-/* Define a function to compare two events to determine which one occurs first
+/*
+ * Define a function to compare two events to determine which one occurs
+ * first.
  */
-
-int determine_event_ordering(Event *e1, Event *e2);
-
-int determine_event_ordering(Event *e1, Event *e2)
+int
+determine_event_ordering(
+	const Event *e1,
+	const Event *e2
+	)
 {
-    return (e1->time - e2->time);
+	return (e1->time - e2->time);
 }
 
-/* Define a function to compare two received packets to determine which one
- * is received first
+
+/*
+ * Define a function to compare two received packets to determine which
+ * one is received first.
  */
-int determine_recv_buf_ordering(struct recvbuf *b1, struct recvbuf *b2);
-
-int determine_recv_buf_ordering(struct recvbuf *b1, struct recvbuf *b2)
+int
+determine_recv_buf_ordering(
+	const struct recvbuf *b1,
+	const struct recvbuf *b2
+	)
 {
-    double recv_time1, recv_time2;
+	double recv_time1;
+	double recv_time2;
 
-    /* Simply convert the time received to double and subtract */
-    LFPTOD(&b1->recv_time, recv_time1);
-    LFPTOD(&b2->recv_time, recv_time2);
-    return ((int)(recv_time1 - recv_time2));
+	/* Simply convert the time received to double and subtract */
+	LFPTOD(&b1->recv_time, recv_time1);
+	LFPTOD(&b2->recv_time, recv_time2);
+
+	return (int)(recv_time1 - recv_time2);
 }
 
+
 /* Define a function to create the server associations */
-void create_server_associations()
+void create_server_associations(void)
 {
-    int i;
-    for (i = 0;i < simulation.num_of_servers;++i) {
-	printf("%s\n", stoa(simulation.servers[i].addr));
-	if (peer_config(simulation.servers[i].addr,
-			ANY_INTERFACE_CHOOSE(simulation.servers[i].addr),
-			MODE_CLIENT,
-			NTP_VERSION,
-			NTP_MINDPOLL,
-			NTP_MAXDPOLL,
-			0, /* peerflags */
-			0, /* ttl */
-			0, /* peerkey */
-			(u_char *)"*" /* peerkeystr */) == 0) {
-	    fprintf(stderr, "ERROR!! Could not create association for: %s",
-		    stoa(simulation.servers[i].addr));
+	int i;
+
+	for (i = 0; i < simulation.num_of_servers; ++i) {
+		printf("%s\n", stoa(simulation.servers[i].addr));
+		if (peer_config(simulation.servers[i].addr,
+				NULL,
+				loopback_interface,
+				MODE_CLIENT,
+				NTP_VERSION,
+				NTP_MINDPOLL,
+				NTP_MAXDPOLL,
+				0, /* peerflags */
+				0, /* ttl */
+				0, /* peerkey */
+				NULL /* group ident */) == 0) {
+			fprintf(stderr,
+				"ERROR!! Could not create association for: %s\n",
+				stoa(simulation.servers[i].addr));
+		}
 	}
-    }
 }
 
 
 /* Main Simulator Code */
 
-int ntpsim(int argc, char *argv[])
+int
+ntpsim(
+	int	argc,
+	char *	argv[]
+	)
 {
-    Event *curr_event;
-    struct timeval seed;
+	Event *		curr_event;
+	struct timeval	seed;
 
-    /* Initialize the local Clock 
-     */
-    simclock.local_time = 0;
-    simclock.adj = 0;
-    simclock.slew = 0;
+	/* Initialize the local Clock */
+	simclock.local_time = 0;
+	simclock.adj = 0;
+	simclock.slew = 500e-6;
 
-    /* Initialize the simulation 
-     */
-    simulation.num_of_servers = 0;
-    simulation.beep_delay = BEEP_DLY;
-    simulation.sim_time = 0;
-    simulation.end_time = SIM_TIME;
+	/* Initialize the simulation */
+	simulation.num_of_servers = 0;
+	simulation.beep_delay = BEEP_DLY;
+	simulation.sim_time = 0;
+	simulation.end_time = SIM_TIME;
 
-    /*
-     * Initialize ntp variables
-     */
-    initializing = 1;
-    init_auth();
-    init_util();
-    init_restrict();
-    init_mon();
-    init_timer();
-    init_lib();
-    init_request();
-    init_control();
-    init_peer();
-    init_proto();
-    init_io();
-    init_loopfilter();
-    mon_start(MON_OFF);    
+	/* Initialize ntp modules */
+	initializing = TRUE;
+	msyslog_term = TRUE;
+	init_sim_io();
+	init_auth();
+	init_util();
+	init_restrict();
+	init_mon();
+	init_timer();
+	init_lib();
+	init_request();
+	init_control();
+	init_peer();
+	init_proto();
+	init_loopfilter();
+	mon_start(MON_OFF);
 
-    /* Call getconfig to parse the configuration file */
-    getconfig(argc, argv);
-    initializing = 0;
-    loop_config(LOOP_DRIFTCOMP, old_drift / 1e6);
+	/* Call getconfig to parse the configuration file */
+	getconfig(argc, argv);
+	loop_config(LOOP_DRIFTINIT, 0);
+	initializing = FALSE;
 
-    /*
-     * Watch out here, we want the real time, not the silly stuff.
-     */
-    gettimeofday(&seed, NULL);
-    ntp_srandom(seed.tv_usec);
+	/*
+	 * Watch out here, we want the real time, not the silly stuff.
+	 */
+	gettimeofday(&seed, NULL);
+	ntp_srandom(seed.tv_usec);
 
+	/* Initialize the event queue */
+	event_queue = create_priority_queue((q_order_func)
+	    determine_event_ordering);
 
-    /* Initialize the event queue */
-    event_queue = create_priority_queue((int(*)(void *, void*)) 
-					determine_event_ordering);
+	/* Initialize the receive queue */
+	recv_queue = create_priority_queue((q_order_func)
+	    determine_recv_buf_ordering);
 
-    /* Initialize the receive queue */
-    recv_queue = create_priority_queue((int(*)(void *, void*))
-				       determine_recv_buf_ordering);
+	/* Push a beep and a timer on the event queue */
+	enqueue(event_queue, event(0, BEEP));
+	enqueue(event_queue, event(simulation.sim_time + 1.0, TIMER));
 
-    /* Push a beep and a timer on the event queue */
-    enqueue(event_queue, event(0, BEEP));
-    enqueue(event_queue, event(simulation.sim_time + 1.0, TIMER));
-    /* 
-     * Pop the queue until nothing is left or time is exceeded
-     */
-    /* maxtime = simulation.sim_time + simulation.end_time;*/
-    while (simulation.sim_time <= simulation.end_time &&
+	/* 
+	 * Pop the queue until nothing is left or time is exceeded
+	 */
+	/* maxtime = simulation.sim_time + simulation.end_time;*/
+	while (simulation.sim_time <= simulation.end_time &&
 	   (!empty(event_queue))) {
-	curr_event = dequeue(event_queue);
-	/* Update all the clocks to the time on the event */
-	sim_update_clocks(curr_event);
+		curr_event = dequeue(event_queue);
+		/* Update all the clocks to the time on the event */
+		sim_update_clocks(curr_event);
 
-	/* Execute the function associated with the event */
-	event_ptr[curr_event->function](curr_event);
-	free_node(curr_event);
-    }
-    return (0);
+		/* Execute the function associated with the event */
+		(*event_ptr[curr_event->function])(curr_event);
+		free_node(curr_event);
+	}
+	printf("sys_received: %lu\n", sys_received);
+	printf("sys_badlength: %lu\n", sys_badlength);
+	printf("sys_declined: %lu\n", sys_declined);
+	printf("sys_restricted: %lu\n", sys_restricted);
+	printf("sys_newversion: %lu\n", sys_newversion);
+	printf("sys_oldversion: %lu\n", sys_oldversion);
+	printf("sys_limitrejected: %lu\n", sys_limitrejected);
+	printf("sys_badauth: %lu\n", sys_badauth);
+
+	return (0);
 }
 
 
+void
+init_sim_io(void)
+{
+	loopback_interface = emalloc_zero(sizeof(*loopback_interface));
+	ep_list = loopback_interface;
+	strlcpy(loopback_interface->name, "IPv4loop",
+		sizeof(loopback_interface->name));
+	loopback_interface->flags = INT_UP | INT_LOOPBACK;
+	loopback_interface->fd = -1;
+	loopback_interface->bfd = -1;
+	loopback_interface->ifnum = 1;
+	loopback_interface->family = AF_INET;
+	AF(&loopback_interface->sin) = AF_INET;
+	SET_ADDR4(&loopback_interface->sin, LOOPBACKADR);
+	SET_PORT(&loopback_interface->sin, NTP_PORT);
+	AF(&loopback_interface->mask) = AF_INET;
+	SET_ADDR4(&loopback_interface->mask, LOOPNETMASK);
+}
+
 
 /* Define a function to create an return an Event  */
 
@@ -189,7 +236,7 @@ void sim_event_timer(Event *e)
     /* Process received buffers */
     while (!empty(recv_queue)) {
 	rbuf = (struct recvbuf *)dequeue(recv_queue);
-	(rbuf->receiver)(rbuf);
+	(*rbuf->receiver)(rbuf);
 	free_node(rbuf);
     }
 
@@ -205,30 +252,30 @@ void sim_event_timer(Event *e)
  * creates a reply packet and pushes the reply packet onto the event queue
  */
 int simulate_server(
-    sockaddr_u *serv_addr,		/* Address of the server */
-    struct interface *inter,		/* Interface on which the reply should
-					   be inserted */
-    struct pkt *rpkt			/* Packet sent to the server that
-					   needs to be processed. */
-)
+    sockaddr_u *serv_addr,	/* Address of the server */
+    endpt *	inter,		/* Interface on which the reply should
+				   be inserted */
+    struct pkt *rpkt		/* Packet sent to the server that
+				   needs to be processed. */
+    )
 {
-    struct pkt xpkt;	       /* Packet to be transmitted back
-				  to the client */
-    struct recvbuf rbuf;       /* Buffer for the received packet */
-    Event *e;		       /* Packet receive event */
-    server_info *server;       /* Pointer to the server being simulated */
-    script_info *curr_script;  /* Current script being processed */
+    struct pkt xpkt;		/* Packet to be transmitted back
+				   to the client */
+    struct recvbuf rbuf;	/* Buffer for the received packet */
+    Event *e;			/* Packet receive event */
+    server_info *server;	/* Pointer to the server being simulated */
+    script_info *curr_script;	/* Current script being processed */
     int i;
-    double d1, d2, d3;	       /* Delays while the packet is enroute */
-    double t1, t2, t3, t4;     /* The four timestamps in the packet */
+    double d1, d2, d3;		/* Delays while the packet is enroute */
+    double t1, t2, t3, t4;	/* The four timestamps in the packet */
+    l_fp lfp_host;		/* host-order l_fp */
 
-    memset(&xpkt, 0, sizeof(xpkt));
-    memset(&rbuf, 0, sizeof(rbuf));
+    ZERO(xpkt);
+    ZERO(rbuf);
 
     /* Search for the server with the desired address */
     server = NULL;
     for (i = 0; i < simulation.num_of_servers; ++i) {
-	fprintf(stderr,"Checking address: %s\n", stoa(simulation.servers[i].addr));
 	if (memcmp(simulation.servers[i].addr, serv_addr, 
 		   sizeof(*serv_addr)) == 0) { 
 	    server = &simulation.servers[i];
@@ -236,7 +283,8 @@ int simulate_server(
 	}
     }
 
-    fprintf(stderr, "Received packet for: %s\n", stoa(serv_addr));
+    fprintf(stderr, "Received packet from %s on %s\n",
+	    stoa(serv_addr), latoa(inter));
     if (server == NULL)
 	abortsim("Server with specified address not found!!!");
     
@@ -247,7 +295,7 @@ int simulate_server(
      * Masquerade the reply as a stratum-1 server with a GPS clock
      */
     xpkt.li_vn_mode = PKT_LI_VN_MODE(LEAP_NOWARNING, NTP_VERSION,
-                                     MODE_SERVER);
+				     MODE_SERVER);
     xpkt.stratum = STRATUM_TO_PKT(((u_char)1));
     memcpy(&xpkt.refid, "GPS", 4);
     xpkt.ppoll = rpkt->ppoll;
@@ -275,54 +323,53 @@ int simulate_server(
      * Note: This function is called at time t1. 
      */
 
-    LFPTOD(&rpkt->xmt, t1);
+    NTOHL_FP(&rpkt->xmt, &lfp_host);
+    LFPTOD(&lfp_host, t1);
     t2 = server->server_time + d1;
     t3 = server->server_time + d1 + d2;
     t4 = t1 + d1 + d2 + d3;
 
     /* Save the timestamps */
-    xpkt.org = rpkt->xmt;     
-    DTOLFP(t2, &xpkt.rec);
-    DTOLFP(t3, &xpkt.xmt);
+    xpkt.org = rpkt->xmt;
+    DTOLFP(t2, &lfp_host);
+    HTONL_FP(&lfp_host, &xpkt.rec);
+    DTOLFP(t3, &lfp_host);
+    HTONL_FP(&lfp_host, &xpkt.xmt);
     xpkt.reftime = xpkt.xmt;
 
-
-
-    /* Ok, we are done with the packet. Now initialize the receive buffer for
-     * the packet.
+    /* 
+     * Ok, we are done with the packet. Now initialize the receive
+     * buffer for the packet.
      */
-    rbuf.receiver = receive;   /* Function to call to process the packet */
+    rbuf.used = 1;
+    rbuf.receiver = &receive;   /* callback to process the packet */
     rbuf.recv_length = LEN_PKT_NOMAC;
     rbuf.recv_pkt = xpkt;
-    rbuf.used = 1;
-
+    rbuf.dstadr = inter;
+    rbuf.fd = inter->fd;
     memcpy(&rbuf.srcadr, serv_addr, sizeof(rbuf.srcadr));
     memcpy(&rbuf.recv_srcadr, serv_addr, sizeof(rbuf.recv_srcadr));
-    if ((rbuf.dstadr = malloc(sizeof(*rbuf.dstadr))) == NULL)
-	abortsim("malloc failed in simulate_server");
-    memcpy(rbuf.dstadr, inter, sizeof(*rbuf.dstadr));
-    /* rbuf.link = NULL; */
 
-    /* Create a packet event and insert it onto the event_queue at the 
+    /*
+     * Create a packet event and insert it onto the event_queue at the
      * arrival time (t4) of the packet at the client 
      */
     e = event(t4, PACKET);
     e->rcv_buf = rbuf;
     enqueue(event_queue, e);
-    
 
-    /* Check if the time of the script has expired. If yes, delete the script.
-     * If not, re-enqueue the script onto the server script queue 
+    /*
+     * Check if the time of the script has expired. If yes, delete it.
      */
     if (curr_script->duration > simulation.sim_time && 
-	!empty(server->script)) {
+	NULL == HEAD_PFIFO(server->script)) {
 	printf("Hello\n");
 	/* 
 	 * For some reason freeing up the curr_script memory kills the
 	 * simulation. Further debugging is needed to determine why.
-	 * free_node(curr_script);
+	 * free(curr_script);
 	 */
-	curr_script = dequeue(server->script);
+	UNLINK_FIFO(curr_script, *server->script, link);
     }
 
     return (0);
@@ -333,7 +380,7 @@ int simulate_server(
  * Most of the code is modified from the systime.c file by Prof. Mills
  */
 
-void sim_update_clocks (Event *e)
+void sim_update_clocks(Event *e)
 {
     double time_gap;
     double adj;
@@ -342,7 +389,14 @@ void sim_update_clocks (Event *e)
     /* Compute the time between the last update event and this update */
     time_gap = e->time - simulation.sim_time;
 
+    if (time_gap < 0)
+	    printf("WARNING: e->time %.6g comes before sim_time %.6g (gap %+.6g)\n",
+		   e->time, simulation.sim_time, time_gap);
+
     /* Advance the client clock */
+    if (e->time + time_gap < simclock.local_time)
+	    printf("WARNING: e->time + gap %.6g comes before local_time %.6g\n",
+		   e->time + time_gap, simclock.local_time);
     simclock.local_time = e->time + time_gap;
 
     /* Advance the simulation time */
@@ -352,7 +406,7 @@ void sim_update_clocks (Event *e)
      * errors. The random error is a random walk computed as the
      * integral of samples from a Gaussian distribution.
      */
-    for (i = 0;i < simulation.num_of_servers; ++i) {
+    for (i = 0; i < simulation.num_of_servers; ++i) {
 	simulation.servers[i].curr_script->freq_offset +=
 	    gauss(0, time_gap * simulation.servers[i].curr_script->wander);
 
@@ -360,7 +414,6 @@ void sim_update_clocks (Event *e)
 	    (1 + simulation.servers[i].curr_script->freq_offset);
     }
 
-
     /* Perform the adjtime() function. If the adjustment completed
      * in the previous interval, amortize the entire amount; if not,
      * carry the leftover to the next interval.
@@ -371,13 +424,11 @@ void sim_update_clocks (Event *e)
 	if (simclock.adj < 0) {
 	    simclock.adj += adj;
 	    simclock.local_time -= adj;
-	} 
-	else {
+	} else {
 	    simclock.adj -= adj;
 	    simclock.local_time += adj;
 	}    
-    } 
-    else {
+    } else {
 	simclock.local_time += simclock.adj;
 	simclock.adj = 0;
     }
@@ -573,7 +624,8 @@ gauss(
      */
     if (s == 0)
         return (m);
-    while ((q1 = drand48()) == 0);
+    while ((q1 = drand48()) == 0)
+	/* empty statement */;
     q2 = drand48();
     return (m + s * sqrt(-2. * log(q1)) * cos(2. * PI * q2));
 }
@@ -597,7 +649,8 @@ poisson(
      */
     if (s == 0)
         return (m);
-    while ((q1 = drand48()) == 0);
+    while ((q1 = drand48()) == 0)
+	/* empty statement */;
     return (m - s * log(q1 * s));
 }
 
