@@ -24,12 +24,6 @@ extern	struct refclock	refclock_local;
 #define	refclock_local	refclock_none
 #endif
 
-#if 0 && defined(CLOCK_TRAK) && defined(PPS)
-extern	struct refclock	refclock_trak;
-#else
-#define	refclock_trak	refclock_none
-#endif
-
 #ifdef CLOCK_PST
 extern	struct refclock	refclock_pst;
 #else
@@ -258,6 +252,17 @@ extern	struct refclock	refclock_neoclock4x;
 #define	refclock_neoclock4x	refclock_none
 #endif
 
+#ifdef CLOCK_TSYNCPCI
+extern struct refclock refclock_tsyncpci;
+#else
+#define refclock_tsyncpci refclock_none
+#endif
+
+#if defined(CLOCK_GPSDJSON) && !defined(SYS_WINNT) 
+extern struct refclock refclock_gpsdjson;
+#else
+#define refclock_gpsdjson refclock_none
+#endif
 /*
  * Order is clock_start(), clock_shutdown(), clock_poll(),
  * clock_control(), clock_init(), clock_buginfo, clock_flags;
@@ -309,7 +314,9 @@ struct refclock * const refclock_conf[] = {
 	&refclock_tt560,	/* 41 REFCLK_TT560 */
 	&refclock_zyfer,	/* 42 REFCLK_ZYFER */
 	&refclock_ripencc,	/* 43 REFCLK_RIPENCC */
-	&refclock_neoclock4x    /* 44 REFCLK_NEOCLOCK4X */
+	&refclock_neoclock4x,	/* 44 REFCLK_NEOCLOCK4X */
+	&refclock_tsyncpci,	/* 45 REFCLK_TSYNCPCI */
+	&refclock_gpsdjson	/* 46 REFCLK_GPSDJSON */
 };
 
 u_char num_refclock_conf = sizeof(refclock_conf)/sizeof(struct refclock *);
