@@ -1,29 +1,38 @@
 /*
  * ntpdc - control and monitor your ntpd daemon
  */
-
+#include <config.h>
 #include <stdio.h>
 #include <stddef.h>
 #include <ctype.h>
 #include <signal.h>
 #include <setjmp.h>
+#ifdef HAVE_UNISTD_H
+# include <unistd.h>
+#endif
+#ifdef HAVE_FCNTL_H
+# include <fcntl.h>
+#endif
+#ifdef SYS_WINNT
+# include <mswsock.h>
+#endif
+#include <isc/net.h>
+#include <isc/result.h>
 
 #include "ntpdc.h"
 #include "ntp_select.h"
-#include "ntp_io.h"
 #include "ntp_stdlib.h"
 #include "ntp_assert.h"
 #include "ntp_lineedit.h"
-#include "isc/net.h"
-#include "isc/result.h"
+#ifdef OPENSSL
+#include "openssl/evp.h"
+#include "openssl/objects.h"
+#endif
 #include <ssl_applink.c>
 
+#include "ntp_libopts.h"
 #include "ntpdc-opts.h"
-
-#ifdef SYS_WINNT
-# include <Mswsock.h>
-# include <io.h>
-#endif /* SYS_WINNT */
+#include "safecast.h"
 
 #ifdef SYS_VXWORKS
 				/* vxWorks needs mode flag -casey*/
@@ -66,8 +75,8 @@ int		ntpdcmain	(int,	char **);
 static	int	openhost	(const char *);
 static	int	sendpkt		(void *, size_t);
 static	void	growpktdata	(void);
-static	int	getresponse	(int, int, int *, int *, char **, int);
-static	int	sendrequest	(int, int, int, u_int, size_t, char *);
+static	int	getresponse	(int, int, size_t *, size_t *, const char **, size_t);
+static	int	sendrequest	(int, int, int, size_t, size_t, const char *);
 static	void	getcmds		(void);
 static	RETSIGTYPE abortcmd	(int);
 static	void	docmd		(const char *);
@@ -76,11 +85,7 @@ static	int	findcmd		(char *, struct xcmd *, struct xcmd *, struct xcmd **);
 static	int	getarg		(char *, int, arg_v *);
 static	int	getnetnum	(const char *, sockaddr_u *, char *, int);
 static	void	help		(struct parse *, FILE *);
-#ifdef QSORT_USES_VOID_P
 static	int	helpsort	(const void *, const void *);
-#else
-static	int	helpsort	(char **, char **);
-#endif
 static	void	printusage	(struct xcmd *, FILE *);
 static	void	timeout		(struct parse *, FILE *);
 static	void	my_delay	(struct parse *, FILE *);
@@ -92,8 +97,10 @@ static	void	hostnames	(struct parse *, FILE *);
 static	void	setdebug	(struct parse *, FILE *);
 static	void	quit		(struct parse *, FILE *);
 static	void	version		(struct parse *, FILE *);
-static	void	warning		(const char *, const char *, const char *);
-static	void	error		(const char *, const char *, const char *);
+static	void	warning		(const char *, ...)
+    __attribute__((__format__(__printf__, 1, 2)));
+static	void	error		(const char *, ...)
+    __attribute__((__format__(__printf__, 1, 2)));
 static	u_long	getkeyid	(const char *);
 
 
@@ -218,11 +225,6 @@ static	const char *chosts[MAXHOSTS];
 #define	ISEOL(c)	((c) == '\n' || (c) == '\r' || (c) == '\0')
 #define	STREQ(a, b)	(*(a) == *(b) && strcmp((a), (b)) == 0)
 
-/*
- * For converting time stamps to dates
- */
-#define	JAN_1970	2208988800	/* 1970 - 1900 in seconds */
-
 /*
  * Jump buffer for longjumping back to the command level
  */
@@ -239,8 +241,7 @@ static	FILE *current_output;
  */
 extern struct xcmd opcmds[];
 
-char *progname;
-volatile int debug;
+char const *progname;
 
 #ifdef NO_MAIN_ALLOWED
 CALL(ntpdc,"ntpdc",ntpdcmain);
@@ -274,7 +275,6 @@ ntpdcmain(
 	char *argv[]
 	)
 {
-	extern int ntp_optind;
 
 	delay_time.l_ui = 0;
 	delay_time.l_uf = DEFDELAY;
@@ -286,6 +286,7 @@ ntpdcmain(
 
 	init_lib();	/* sets up ipv4_works, ipv6_works */
 	ssl_applink();
+	init_auth();
 
 	/* Check to see if we have IPv6. Otherwise default to IPv4 */
 	if (!ipv6_works)
@@ -294,7 +295,7 @@ ntpdcmain(
 	progname = argv[0];
 
 	{
-		int optct = optionProcess(&ntpdcOptions, argc, argv);
+		int optct = ntpOptionProcess(&ntpdcOptions, argc, argv);
 		argc -= optct;
 		argv += optct;
 	}
@@ -315,7 +316,7 @@ ntpdcmain(
 		}
 	}
 
-	debug = DESC(DEBUG_LEVEL).optOccCt;
+	debug = OPT_VALUE_SET_DEBUG_LEVEL;
 
 	if (HAVE_OPT(INTERACTIVE)) {
 		interactive = 1;
@@ -349,62 +350,6 @@ ntpdcmain(
 		interactive = 1;
 	}
 
-#if 0
-	ai_fam_templ = ai_fam_default;
-	while ((c = ntp_getopt(argc, argv, "46c:dilnps")) != EOF)
-	    switch (c) {
-		case '4':
-		    ai_fam_templ = AF_INET;
-		    break;
-		case '6':
-		    ai_fam_templ = AF_INET6;
-		    break;
-		case 'c':
-		    ADDCMD(ntp_optarg);
-		    break;
-		case 'd':
-		    ++debug;
-		    break;
-		case 'i':
-		    interactive = 1;
-		    break;
-		case 'l':
-		    ADDCMD("listpeers");
-		    break;
-		case 'n':
-		    showhostnames = 0;
-		    break;
-		case 'p':
-		    ADDCMD("peers");
-		    break;
-		case 's':
-		    ADDCMD("dmpeers");
-		    break;
-		default:
-		    errflg++;
-		    break;
-	    }
-
-	if (errflg) {
-		(void) fprintf(stderr,
-			       "usage: %s [-46dilnps] [-c cmd] host ...\n",
-			       progname);
-		exit(2);
-	}
-
-	if (ntp_optind == argc) {
-		ADDHOST(DEFHOST);
-	} else {
-		for (; ntp_optind < argc; ntp_optind++)
-		    ADDHOST(argv[ntp_optind]);
-	}
-
-	if (numcmds == 0 && interactive == 0
-	    && isatty(fileno(stdin)) && isatty(fileno(stderr))) {
-		interactive = 1;
-	}
-#endif
-
 #ifndef SYS_WINNT /* Under NT cannot handle SIGINT, WIN32 spawns a handler */
 	if (interactive)
 	    (void) signal_no_reset(SIGINT, abortcmd);
@@ -450,6 +395,8 @@ openhost(
 	char temphost[LENHOSTNAME];
 	int a_info, i;
 	struct addrinfo hints, *ai = NULL;
+	sockaddr_u addr;
+	size_t octets;
 	register const char *cp;
 	char name[LENHOSTNAME];
 	char service[5];
@@ -479,12 +426,12 @@ openhost(
 	 * will return an "IPv4-mapped IPv6 address" address if you
 	 * give it an IPv4 address to lookup.
 	 */
-	strcpy(service, "ntp");
-	memset((char *)&hints, 0, sizeof(struct addrinfo));
+	strlcpy(service, "ntp", sizeof(service));
+	ZERO(hints);
 	hints.ai_family = ai_fam_templ;
 	hints.ai_protocol = IPPROTO_UDP;
 	hints.ai_socktype = SOCK_DGRAM;
-	hints.ai_flags = AI_NUMERICHOST;
+	hints.ai_flags = Z_AI_NUMERICHOST;
 
 	a_info = getaddrinfo(hname, service, &hints, &ai);
 	if (a_info == EAI_NONAME
@@ -504,7 +451,7 @@ openhost(
 		a_info = getaddrinfo(hname, service, &hints, &ai);	
 	}
 	if (a_info != 0) {
-		(void) fprintf(stderr, "%s\n", gai_strerror(a_info));
+		fprintf(stderr, "%s\n", gai_strerror(a_info));
 		if (ai != NULL)
 			freeaddrinfo(ai);
 		return 0;
@@ -514,30 +461,29 @@ openhost(
 	 * getaddrinfo() has returned without error so ai should not 
 	 * be NULL.
 	 */
-	NTP_INSIST(ai != NULL);
+	INSIST(ai != NULL);
+	ZERO(addr);
+	octets = min(sizeof(addr), ai->ai_addrlen);
+	memcpy(&addr, ai->ai_addr, octets);
 
-	if (ai->ai_canonname == NULL) {
-		strncpy(temphost, stoa((sockaddr_u *)ai->ai_addr),
-		    LENHOSTNAME);
-		temphost[LENHOSTNAME-1] = '\0';
-	} else {
-		strncpy(temphost, ai->ai_canonname, LENHOSTNAME);
-		temphost[LENHOSTNAME-1] = '\0';
-	}
+	if (ai->ai_canonname == NULL)
+		strlcpy(temphost, stoa(&addr), sizeof(temphost));
+	else
+		strlcpy(temphost, ai->ai_canonname, sizeof(temphost));
 
 	if (debug > 2)
-	    printf("Opening host %s\n", temphost);
+		printf("Opening host %s\n", temphost);
 
 	if (havehost == 1) {
 		if (debug > 2)
-		    printf("Closing old host %s\n", currenthost);
-		(void) closesocket(sockfd);
+			printf("Closing old host %s\n", currenthost);
+		closesocket(sockfd);
 		havehost = 0;
 	}
-	(void) strcpy(currenthost, temphost);
+	strlcpy(currenthost, temphost, sizeof(currenthost));
 	
 	/* port maps to the same in both families */
-	s_port = ((struct sockaddr_in6 *)ai->ai_addr)->sin6_port; 
+	s_port = NSRCPORT(&addr);; 
 #ifdef SYS_VXWORKS
 	((struct sockaddr_in6 *)&hostaddr)->sin6_port = htons(SERVER_PORT_NUM);
 	if (ai->ai_family == AF_INET)
@@ -559,18 +505,13 @@ openhost(
 			exit(1);
 		}
 	}
+#endif /* SYS_WINNT */
 
 	sockfd = socket(ai->ai_family, SOCK_DGRAM, 0);
 	if (sockfd == INVALID_SOCKET) {
-		error("socket", "", "");
+		error("socket");
 		exit(-1);
 	}
-#else
-	sockfd = socket(ai->ai_family, SOCK_DGRAM, 0);
-	if (sockfd == -1)
-	    error("socket", "", "");
-#endif /* SYS_WINNT */
-
 	
 #ifdef NEED_RCVBUF_SLOP
 # ifdef SO_RCVBUF
@@ -579,7 +520,7 @@ openhost(
 
 		if (setsockopt(sockfd, SOL_SOCKET, SO_RCVBUF,
 			       &rbufsize, sizeof(int)) == -1)
-		    error("setsockopt", "", "");
+		    error("setsockopt");
 	}
 # endif
 #endif
@@ -588,10 +529,12 @@ openhost(
 	if (connect(sockfd, (struct sockaddr *)&hostaddr, 
 		    sizeof(hostaddr)) == -1)
 #else
-	if (connect(sockfd, (struct sockaddr *)ai->ai_addr,
-		    ai->ai_addrlen) == -1)
+	if (connect(sockfd, ai->ai_addr, ai->ai_addrlen) == -1)
 #endif /* SYS_VXWORKS */
-	    error("connect", "", "");
+	{
+		error("connect");
+		exit(-1);
+	}
 
 	freeaddrinfo(ai);
 	havehost = 1;
@@ -612,7 +555,7 @@ sendpkt(
 	)
 {
 	if (send(sockfd, xdata, xdatalen, 0) == -1) {
-		warning("write to %s failed", currenthost, "");
+		warning("write to %s failed", currenthost);
 		return -1;
 	}
 
@@ -626,8 +569,11 @@ sendpkt(
 static void
 growpktdata(void)
 {
+	size_t priorsz;
+
+	priorsz = (size_t)pktdatasize;
 	pktdatasize += INCDATASIZE;
-	pktdata = erealloc(pktdata, (size_t)pktdatasize);
+	pktdata = erealloc_zero(pktdata, (size_t)pktdatasize, priorsz);
 }
 
 
@@ -638,18 +584,18 @@ static int
 getresponse(
 	int implcode,
 	int reqcode,
-	int *ritems,
-	int *rsize,
-	char **rdata,
-	int esize
+	size_t *ritems,
+	size_t *rsize,
+	const char **rdata,
+	size_t esize
 	)
 {
 	struct resp_pkt rpkt;
 	struct sock_timeval tvo;
-	int items;
-	int i;
-	int size;
-	int datasize;
+	size_t items;
+	size_t i;
+	size_t size;
+	size_t datasize;
 	char *datap;
 	char *tmp_data;
 	char haveseq[MAXSEQ+1];
@@ -658,8 +604,12 @@ getresponse(
 	int numrecv;
 	int seq;
 	fd_set fds;
-	int n;
+	ssize_t n;
 	int pad;
+	/* absolute timeout checks. Not 'time_t' by intention! */
+	uint32_t tobase;	/* base value for timeout */
+	uint32_t tospan;	/* timeout span (max delay) */
+	uint32_t todiff;	/* current delay */
 
 	/*
 	 * This is pretty tricky.  We may get between 1 and many packets
@@ -674,29 +624,42 @@ getresponse(
 	numrecv = 0;
 	*rdata = datap = pktdata;
 	lastseq = 999;	/* too big to be a sequence number */
-	memset(haveseq, 0, sizeof(haveseq));
+	ZERO(haveseq);
 	FD_ZERO(&fds);
+	tobase = (uint32_t)time(NULL);
 
     again:
 	if (firstpkt)
 		tvo = tvout;
 	else
 		tvo = tvsout;
+	tospan = (uint32_t)tvo.tv_sec + (tvo.tv_usec != 0);
 	
 	FD_SET(sockfd, &fds);
-	n = select(sockfd+1, &fds, (fd_set *)0, (fd_set *)0, &tvo);
-
+	n = select(sockfd+1, &fds, NULL, NULL, &tvo);
 	if (n == -1) {
-		warning("select fails", "", "");
+		warning("select fails");
 		return -1;
 	}
+	
+	/*
+	 * Check if this is already too late. Trash the data and fake a
+	 * timeout if this is so.
+	 */
+	todiff = (((uint32_t)time(NULL)) - tobase) & 0x7FFFFFFFu;
+	if ((n > 0) && (todiff > tospan)) {
+		n = recv(sockfd, (char *)&rpkt, sizeof(rpkt), 0);
+		n = 0; /* faked timeout return from 'select()'*/
+	}
+	
 	if (n == 0) {
 		/*
 		 * Timed out.  Return what we have
 		 */
 		if (firstpkt) {
 			(void) fprintf(stderr,
-				       "%s: timed out, nothing received\n", currenthost);
+				       "%s: timed out, nothing received\n",
+				       currenthost);
 			return ERR_TIMEOUT;
 		} else {
 			(void) fprintf(stderr,
@@ -706,7 +669,7 @@ getresponse(
 				printf("Received sequence numbers");
 				for (n = 0; n <= MAXSEQ; n++)
 				    if (haveseq[n])
-					printf(" %d,", n);
+					printf(" %zd,", (size_t)n);
 				if (lastseq != 999)
 				    printf(" last frame received\n");
 				else
@@ -718,7 +681,7 @@ getresponse(
 
 	n = recv(sockfd, (char *)&rpkt, sizeof(rpkt), 0);
 	if (n == -1) {
-		warning("read", "", "");
+		warning("read");
 		return -1;
 	}
 
@@ -726,37 +689,37 @@ getresponse(
 	/*
 	 * Check for format errors.  Bug proofing.
 	 */
-	if (n < RESP_HEADER_SIZE) {
+	if (n < (ssize_t)RESP_HEADER_SIZE) {
 		if (debug)
-		    printf("Short (%d byte) packet received\n", n);
+			printf("Short (%zd byte) packet received\n", (size_t)n);
 		goto again;
 	}
 	if (INFO_VERSION(rpkt.rm_vn_mode) > NTP_VERSION ||
 	    INFO_VERSION(rpkt.rm_vn_mode) < NTP_OLDVERSION) {
 		if (debug)
-		    printf("Packet received with version %d\n",
-			   INFO_VERSION(rpkt.rm_vn_mode));
+			printf("Packet received with version %d\n",
+			       INFO_VERSION(rpkt.rm_vn_mode));
 		goto again;
 	}
 	if (INFO_MODE(rpkt.rm_vn_mode) != MODE_PRIVATE) {
 		if (debug)
-		    printf("Packet received with mode %d\n",
-			   INFO_MODE(rpkt.rm_vn_mode));
+			printf("Packet received with mode %d\n",
+			       INFO_MODE(rpkt.rm_vn_mode));
 		goto again;
 	}
 	if (INFO_IS_AUTH(rpkt.auth_seq)) {
 		if (debug)
-		    printf("Encrypted packet received\n");
+			printf("Encrypted packet received\n");
 		goto again;
 	}
 	if (!ISRESPONSE(rpkt.rm_vn_mode)) {
 		if (debug)
-		    printf("Received request packet, wanted response\n");
+			printf("Received request packet, wanted response\n");
 		goto again;
 	}
 	if (INFO_MBZ(rpkt.mbz_itemsize) != 0) {
 		if (debug)
-		    printf("Received packet with nonzero MBZ field!\n");
+			printf("Received packet with nonzero MBZ field!\n");
 		goto again;
 	}
 
@@ -765,7 +728,7 @@ getresponse(
 	 */
 	if (rpkt.implementation != implcode || rpkt.request != reqcode) {
 		if (debug)
-		    printf(
+			printf(
 			    "Received implementation/request of %d/%d, wanted %d/%d",
 			    rpkt.implementation, rpkt.request,
 			    implcode, reqcode);
@@ -796,8 +759,8 @@ getresponse(
 	if ((size_t)datasize > (n-RESP_HEADER_SIZE)) {
 		if (debug)
 		    printf(
-			    "Received items %d, size %d (total %d), data in packet is %lu\n",
-			    items, size, datasize, (u_long)(n-RESP_HEADER_SIZE));
+			    "Received items %zu, size %zu (total %zu), data in packet is %zu\n",
+			    items, size, datasize, n-RESP_HEADER_SIZE);
 		goto again;
 	}
 
@@ -805,9 +768,9 @@ getresponse(
 	 * If this isn't our first packet, make sure the size matches
 	 * the other ones.
 	 */
-	if (!firstpkt && esize != *rsize) {
+	if (!firstpkt && size != *rsize) {
 		if (debug)
-		    printf("Received itemsize %d, previous %d\n",
+		    printf("Received itemsize %zu, previous %zu\n",
 			   size, *rsize);
 		goto again;
 	}
@@ -834,10 +797,12 @@ getresponse(
 	}
 
 	/*
-	 * So far, so good.  Copy this data into the output array.
+	 * So far, so good.  Copy this data into the output array. Bump
+	 * the timeout base, in case we expect more data.
 	 */
+	tobase = (uint32_t)time(NULL);
 	if ((datap + datasize + (pad * items)) > (pktdata + pktdatasize)) {
-		int offset = datap - pktdata;
+		size_t offset = datap - pktdata;
 		growpktdata();
 		*rdata = pktdata; /* might have been realloced ! */
 		datap = pktdata + offset;
@@ -847,11 +812,11 @@ getresponse(
 	 * items.  This is so we can play nice with older implementations
 	 */
 
-	tmp_data = rpkt.data;
+	tmp_data = rpkt.u.data;
 	for (i = 0; i < items; i++) {
 		memcpy(datap, tmp_data, (unsigned)size);
 		tmp_data += size;
-		memset(datap + size, 0, pad);
+		zero_mem(datap + size, pad);
 		datap += size + pad;
 	}
 
@@ -900,9 +865,9 @@ sendrequest(
 	int implcode,
 	int reqcode,
 	int auth,
-	u_int qitems,
+	size_t qitems,
 	size_t qsize,
-	char *qdata
+	const char *qdata
 	)
 {
 	struct req_pkt qpkt;
@@ -911,18 +876,17 @@ sendrequest(
 	u_long	key_id;
 	l_fp	ts;
 	l_fp *	ptstamp;
-	int	maclen;
+	size_t	maclen;
 	char *	pass;
 
-	memset(&qpkt, 0, sizeof(qpkt));
-
+	ZERO(qpkt);
 	qpkt.rm_vn_mode = RM_VN_MODE(0, 0, 0);
 	qpkt.implementation = (u_char)implcode;
 	qpkt.request = (u_char)reqcode;
 
 	datasize = qitems * qsize;
 	if (datasize && qdata != NULL) {
-		memcpy(qpkt.data, qdata, datasize);
+		memcpy(qpkt.u.data, qdata, datasize);
 		qpkt.err_nitems = ERR_NITEMS(0, qitems);
 		qpkt.mbz_itemsize = MBZ_ITEMSIZE(qsize);
 	} else {
@@ -975,15 +939,16 @@ sendrequest(
 	get_systime(&ts);
 	L_ADD(&ts, &delay_time);
 	HTONL_FP(&ts, ptstamp);
-	maclen = authencrypt(info_auth_keyid, (void *)&qpkt, reqsize);
+	maclen = authencrypt(
+		info_auth_keyid, (void *)&qpkt, size2int_chk(reqsize));
 	if (!maclen) {  
 		fprintf(stderr, "Key not found\n");
 		return 1;
-	} else if (maclen != (info_auth_hashlen + sizeof(keyid_t))) {
+	} else if (maclen != (int)(info_auth_hashlen + sizeof(keyid_t))) {
 		fprintf(stderr,
-			"%d octet MAC, %lu expected with %lu octet digest\n",
-			maclen, (u_long)(info_auth_hashlen + sizeof(keyid_t)),
-			(u_long)info_auth_hashlen);
+			"%zu octet MAC, %zu expected with %zu octet digest\n",
+			maclen, (info_auth_hashlen + sizeof(keyid_t)),
+			info_auth_hashlen);
 		return 1;
 	}
 	return sendpkt(&qpkt, reqsize + maclen);
@@ -998,12 +963,12 @@ doquery(
 	int implcode,
 	int reqcode,
 	int auth,
-	int qitems,
-	int qsize,
-	char *qdata,
-	int *ritems,
-	int *rsize,
-	char **rdata,
+	size_t qitems,
+	size_t qsize,
+	const char *qdata,
+	size_t *ritems,
+	size_t *rsize,
+	const char **rdata,
  	int quiet_mask,
 	int esize
 	)
@@ -1029,10 +994,9 @@ again:
 		tvzero.tv_sec = tvzero.tv_usec = 0;
 		FD_ZERO(&fds);
 		FD_SET(sockfd, &fds);
-		res = select(sockfd+1, &fds, (fd_set *)0, (fd_set *)0, &tvzero);
-
+		res = select(sockfd+1, &fds, NULL, NULL, &tvzero);
 		if (res == -1) {
-			warning("polling select", "", "");
+			warning("polling select");
 			return -1;
 		} else if (res > 0)
 
@@ -1088,7 +1052,7 @@ again:
 			if (implcode == IMPL_XNTPD)
 				break;
 			(void) fprintf(stderr,
-				       "***Server implementation incompatable with our own\n");
+				       "***Server implementation incompatible with our own\n");
 			break;
 		case INFO_ERR_REQ:
 			(void) fprintf(stderr,
@@ -1182,6 +1146,11 @@ docmd(
 	/*
 	 * Tokenize the command line.  If nothing on it, return.
 	 */
+	if (strlen(cmdline) >= MAXLINE) {
+		fprintf(stderr, "***Command ignored, more than %d characters:\n%s\n",
+			MAXLINE - 1, cmdline);
+		return;
+	}
 	tokenize(cmdline, tokens, &ntok);
 	if (ntok == 0)
 	    return;
@@ -1323,7 +1292,7 @@ findcmd(
 	)
 {
 	register struct xcmd *cl;
-	register int clen;
+	size_t clen;
 	int nmatch;
 	struct xcmd *nearmatch = NULL;
 	struct xcmd *clist;
@@ -1399,8 +1368,7 @@ getarg(
 	char *cp, *np;
 	static const char *digits = "0123456789";
 
-	memset(argp, 0, sizeof(*argp));
-
+	ZERO(*argp);
 	argp->string = str;
 	argp->type   = code & ~OPT;
 
@@ -1437,7 +1405,7 @@ getarg(
 				return 0;
 			}
 			argp->uval *= 10;
-			argp->uval += (cp - digits);
+			argp->uval += (u_long)(cp - digits);
 		} while (*(++np) != '\0');
 
 		if (isneg) {
@@ -1478,51 +1446,58 @@ getnetnum(
 	int af
 	)
 {
-	int sockaddr_len;
 	struct addrinfo hints, *ai = NULL;
 
-	sockaddr_len = SIZEOF_SOCKADDR(af);
-	memset((char *)&hints, 0, sizeof(struct addrinfo));
+	ZERO(hints);
 	hints.ai_flags = AI_CANONNAME;
 #ifdef AI_ADDRCONFIG
 	hints.ai_flags |= AI_ADDRCONFIG;
 #endif
 	
-	/* decodenetnum only works with addresses */
+	/*
+	 * decodenetnum only works with addresses, but handles syntax
+	 * that getaddrinfo doesn't:  [2001::1]:1234
+	 */
 	if (decodenetnum(hname, num)) {
-		if (fullhost != 0) {
-			getnameinfo(&num->sa, sockaddr_len, 
-				    fullhost, sizeof(fullhost), NULL, 0, 
-				    NI_NUMERICHOST); 
-		}
+		if (fullhost != NULL)
+			getnameinfo(&num->sa, SOCKLEN(num), fullhost,
+				    LENHOSTNAME, NULL, 0, 0);
 		return 1;
 	} else if (getaddrinfo(hname, "ntp", &hints, &ai) == 0) {
-		memmove((char *)num, ai->ai_addr, ai->ai_addrlen);
-		if (fullhost != 0)
-			(void) strcpy(fullhost, ai->ai_canonname);
+		INSIST(sizeof(*num) >= ai->ai_addrlen);
+		memcpy(num, ai->ai_addr, ai->ai_addrlen);
+		if (fullhost != NULL) {
+			if (ai->ai_canonname != NULL)
+				strlcpy(fullhost, ai->ai_canonname,
+					LENHOSTNAME);
+			else
+				getnameinfo(&num->sa, SOCKLEN(num),
+					    fullhost, LENHOSTNAME, NULL,
+					    0, 0);
+		}
 		return 1;
-	} else {
-		(void) fprintf(stderr, "***Can't find host %s\n", hname);
-		return 0;
 	}
-	/*NOTREACHED*/
+	fprintf(stderr, "***Can't find host %s\n", hname);
+
+	return 0;
 }
 
+
 /*
  * nntohost - convert network number to host name.  This routine enforces
  *	       the showhostnames setting.
  */
-char *
+const char *
 nntohost(
 	sockaddr_u *netnum
 	)
 {
-	if (!showhostnames)
+	if (!showhostnames || SOCK_UNSPEC(netnum))
 		return stoa(netnum);
-
-	if (ISREFCLOCKADR(netnum))
+	else if (ISREFCLOCKADR(netnum))
 		return refnumtoa(netnum);
-	return socktohost(netnum);
+	else
+		return socktohost(netnum);
 }
 
 
@@ -1542,58 +1517,51 @@ help(
 	struct xcmd *xcp;
 	char *cmd;
 	const char *list[100];
-	int word, words;     
-        int row, rows;
-	int col, cols;
+	size_t word, words;
+	size_t row, rows;
+	size_t col, cols;
+	size_t length;
 
 	if (pcmd->nargs == 0) {
 		words = 0;
 		for (xcp = builtins; xcp->keyword != 0; xcp++) {
 			if (*(xcp->keyword) != '?')
-			    list[words++] = xcp->keyword;
+				list[words++] = xcp->keyword;
 		}
-                for (xcp = opcmds; xcp->keyword != 0; xcp++)
-		    list[words++] = xcp->keyword;
+		for (xcp = opcmds; xcp->keyword != 0; xcp++)
+			list[words++] = xcp->keyword;
 
-		qsort(
-#ifdef QSORT_USES_VOID_P
-		    (void *)
-#else
-		    (char *)
-#endif
-			(list), (size_t)(words), sizeof(char *), helpsort);
+		qsort((void *)list, words, sizeof(list[0]), helpsort);
 		col = 0;
 		for (word = 0; word < words; word++) {
-			int length = strlen(list[word]);
-			if (col < length) {
-			    col = length;
-                        }
+			length = strlen(list[word]);
+			col = max(col, length);
 		}
 
 		cols = SCREENWIDTH / ++col;
-                rows = (words + cols - 1) / cols;
+		rows = (words + cols - 1) / cols;
 
-		(void) fprintf(fp, "ntpdc commands:\n");
+		fprintf(fp, "ntpdc commands:\n");
 
 		for (row = 0; row < rows; row++) {
-                        for (word = row; word < words; word += rows) {
-				(void) fprintf(fp, "%-*.*s", col, col-1, list[word]);
-                        }
-			(void) fprintf(fp, "\n");
+			for (word = row; word < words; word += rows)
+				fprintf(fp, "%-*.*s", (int)col,
+					(int)col - 1, list[word]);
+			fprintf(fp, "\n");
 		}
 	} else {
 		cmd = pcmd->argval[0].string;
 		words = findcmd(cmd, builtins, opcmds, &xcp);
 		if (words == 0) {
-			(void) fprintf(stderr,
-				       "Command `%s' is unknown\n", cmd);
+			fprintf(stderr,
+				"Command `%s' is unknown\n", cmd);
 			return;
 		} else if (words >= 2) {
-			(void) fprintf(stderr,
-				       "Command `%s' is ambiguous\n", cmd);
+			fprintf(stderr,
+				"Command `%s' is ambiguous\n", cmd);
 			return;
 		}
-		(void) fprintf(fp, "function: %s\n", xcp->comment);
+		fprintf(fp, "function: %s\n", xcp->comment);
 		printusage(xcp, fp);
 	}
 }
@@ -1602,28 +1570,17 @@ help(
 /*
  * helpsort - do hostname qsort comparisons
  */
-#ifdef QSORT_USES_VOID_P
 static int
 helpsort(
 	const void *t1,
 	const void *t2
 	)
 {
-	char const * const * name1 = (char const * const *)t1;
-	char const * const * name2 = (char const * const *)t2;
+	const char * const *	name1 = t1;
+	const char * const *	name2 = t2;
 
 	return strcmp(*name1, *name2);
 }
-#else
-static int
-helpsort(
-	char **name1,
-	char **name2
-	)
-{
-	return strcmp(*name1, *name2);
-}
-#endif
 
 
 /*
@@ -1837,20 +1794,17 @@ passwd(
 			return;
 		}
 	}
-	if (!interactive) {
-		authusekey(info_auth_keyid, info_auth_keytype,
-			   (u_char *)pcmd->argval[0].string);
-		authtrust(info_auth_keyid, 1);
-	} else {
+	if (pcmd->nargs >= 1)
+		pass = pcmd->argval[0].string;
+	else {
 		pass = getpass_keytype(info_auth_keytype);
-		if (*pass == '\0')
-		    (void) fprintf(fp, "Password unchanged\n");
-		else {
-		    authusekey(info_auth_keyid, info_auth_keytype,
-			       (u_char *)pass);
-		    authtrust(info_auth_keyid, 1);
+		if ('\0' == *pass) {
+			fprintf(fp, "Password unchanged\n");
+			return;
 		}
 	}
+	authusekey(info_auth_keyid, info_auth_keytype, (u_char *)pass);
+	authtrust(info_auth_keyid, 1);
 }
 
 
@@ -1937,34 +1891,44 @@ version(
 }
 
 
+static void __attribute__((__format__(__printf__, 1, 0)))
+vwarning(const char *fmt, va_list ap)
+{
+	int serrno = errno;
+	(void) fprintf(stderr, "%s: ", progname);
+	vfprintf(stderr, fmt, ap);
+	(void) fprintf(stderr, ": %s\n", strerror(serrno));
+}
+
 /*
  * warning - print a warning message
  */
-static void
+static void __attribute__((__format__(__printf__, 1, 2)))
 warning(
 	const char *fmt,
-	const char *st1,
-	const char *st2
+	...
 	)
 {
-	(void) fprintf(stderr, "%s: ", progname);
-	(void) fprintf(stderr, fmt, st1, st2);
-	(void) fprintf(stderr, ": ");
-	perror("");
+	va_list ap;
+	va_start(ap, fmt);
+	vwarning(fmt, ap);
+	va_end(ap);
 }
 
 
 /*
  * error - print a message and exit
  */
-static void
+static void __attribute__((__format__(__printf__, 1, 2)))
 error(
 	const char *fmt,
-	const char *st1,
-	const char *st2
+	...
 	)
 {
-	warning(fmt, st1, st2);
+	va_list ap;
+	va_start(ap, fmt);
+	vwarning(fmt, ap);
+	va_end(ap);
 	exit(1);
 }
 
@@ -1976,10 +1940,11 @@ getkeyid(
 	const char *keyprompt
 	)
 {
-	register char *p;
-	register int c;
+	int c;
 	FILE *fi;
 	char pbuf[20];
+	size_t i;
+	size_t ilim;
 
 #ifndef SYS_WINNT
 	if ((fi = fdopen(open("/dev/tty", 2), "r")) == NULL)
@@ -1987,15 +1952,16 @@ getkeyid(
 	if ((fi = _fdopen(open("CONIN$", _O_TEXT), "r")) == NULL)
 #endif /* SYS_WINNT */
 		fi = stdin;
-	    else
+	else
 		setbuf(fi, (char *)NULL);
 	fprintf(stderr, "%s", keyprompt); fflush(stderr);
-	for (p=pbuf; (c = getc(fi))!='\n' && c!=EOF;) {
-		if (p < &pbuf[18])
-		    *p++ = (char) c;
-	}
-	*p = '\0';
+	for (i = 0, ilim = COUNTOF(pbuf) - 1;
+	     i < ilim && (c = getc(fi)) != '\n' && c != EOF;
+	     )
+		pbuf[i++] = (char)c;
+	pbuf[i] = '\0';
 	if (fi != stdin)
-	    fclose(fi);
-	return (u_int32)atoi(pbuf);
+		fclose(fi);
+
+	return (u_long) atoi(pbuf);
 }
