@@ -30,34 +30,21 @@ enum {
 	MM_TIMER_HIRES
 };
 
-/*
- * Optional callback from libntp ntp_set_tod() to Windows ntpd code
- * in nt_clockstuff that needs to know.  Optional because other
- * libntp clients like ntpdate don't use it.
- */
-
-typedef void (*time_stepped_callback)(void);
-extern time_stepped_callback	step_callback;
-
 /*
  * get_sys_time_as_filetime is a function pointer to
  * either GetSystemTimeAsFileTime provided by Windows
  * or ntpd's interpolating replacement.
  */
-
-typedef void (WINAPI *PGSTAFT)(LPFILETIME pft);
+typedef void (WINAPI *PGSTAFT)(LPFILETIME pftResult);
 extern PGSTAFT get_sys_time_as_filetime;
+extern PGSTAFT pGetSystemTimePreciseAsFileTime;
+
+/* This should really be in a ntpd_win.h or similar:
+ * Set thread description visible in debugger.
+ */
+typedef HRESULT (WINAPI *PSTD)(HANDLE hThread, PCWSTR pwstr);
+extern PSTD pSetThreadDescription;
 
 void lock_thread_to_processor(HANDLE);
 
-#ifdef HAVE_PPSAPI
-/*
- * ntp_timestamp_from_counter provides an interface for 
- * serialpps.sys counterstamps to be converted to 
- * interpolated timestamps.
- */
-
-extern void ntp_timestamp_from_counter(l_fp *, ULONGLONG, ULONGLONG);
-#endif
-
 #endif
