@@ -88,12 +88,13 @@
 #include <sys/stat.h>
 #include <sys/time.h>
 #include <sys/types.h>
-#include "ntp_types.h"
+
+#include "ntp.h"
 #include "ntp_random.h"
 #include "ntp_stdlib.h"
 #include "ntp_assert.h"
-
 #include "ntp_libopts.h"
+#include "ntp_unixtime.h"
 #include "ntp-keygen-opts.h"
 
 #ifdef OPENSSL
@@ -104,19 +105,16 @@
 #include "openssl/pem.h"
 #include "openssl/x509v3.h"
 #include <openssl/objects.h>
-#endif /* OPENSSL */
+#endif	/* OPENSSL */
 #include <ssl_applink.c>
 
+#define _UC(str)	((char *)(intptr_t)(str))
 /*
  * Cryptodefines
  */
 #define	MD5KEYS		10	/* number of keys generated of each type */
 #define	MD5SIZE		20	/* maximum key size */
-#define	JAN_1970	2208988800UL /* NTP seconds */
-#define YEAR		((long)60*60*24*365) /* one year in seconds */
-#define MAXFILENAME	256	/* max file name length */
-#define MAXHOSTNAME	256	/* max host name length */
-#ifdef OPENSSL
+#ifdef AUTOKEY
 #define	PLEN		512	/* default prime modulus size (bits) */
 #define	ILEN		256	/* default identity modulus size (bits) */
 #define	MVMAX		100	/* max MV parameters */
@@ -128,73 +126,120 @@
 #define BASIC_CONSTRAINTS	"critical,CA:TRUE"
 #define EXT_KEY_PRIVATE		"private"
 #define EXT_KEY_TRUST		"trustRoot"
-#endif /* OPENSSL */
+#endif	/* AUTOKEY */
 
 /*
  * Prototypes
  */
 FILE	*fheader	(const char *, const char *, const char *);
-int	gen_md5		(char *);
-#ifdef OPENSSL
-EVP_PKEY *gen_rsa	(char *);
-EVP_PKEY *gen_dsa	(char *);
-EVP_PKEY *gen_iffkey	(char *);
-EVP_PKEY *gen_gqkey	(char *);
-EVP_PKEY *gen_mvkey	(char *, EVP_PKEY **);
+int	gen_md5		(const char *);
+void	followlink	(char *, size_t);
+#ifdef AUTOKEY
+EVP_PKEY *gen_rsa	(const char *);
+EVP_PKEY *gen_dsa	(const char *);
+EVP_PKEY *gen_iffkey	(const char *);
+EVP_PKEY *gen_gqkey	(const char *);
+EVP_PKEY *gen_mvkey	(const char *, EVP_PKEY **);
 void	gen_mvserv	(char *, EVP_PKEY **);
-int	x509		(EVP_PKEY *, const EVP_MD *, char *, char *,
+int	x509		(EVP_PKEY *, const EVP_MD *, char *, const char *,
 			    char *);
 void	cb		(int, int, void *);
-EVP_PKEY *genkey	(char *, char *);
+EVP_PKEY *genkey	(const char *, const char *);
 EVP_PKEY *readkey	(char *, char *, u_int *, EVP_PKEY **);
 void	writekey	(char *, char *, u_int *, EVP_PKEY **);
 u_long	asn2ntp		(ASN1_TIME *);
-#endif /* OPENSSL */
+#endif	/* AUTOKEY */
 
 /*
  * Program variables
  */
 extern char *optarg;		/* command line argument */
-char	*progname;
-volatile int	debug = 0;		/* debug, not de bug */
-#ifdef OPENSSL
-u_int	modulus = PLEN;		/* prime modulus size (bits) */
-u_int	modulus2 = ILEN;	/* identity modulus size (bits) */
-#endif
+char	const *progname;
+u_int	lifetime = DAYSPERYEAR;	/* certificate lifetime (days) */
 int	nkeys;			/* MV keys */
 time_t	epoch;			/* Unix epoch (seconds) since 1970 */
 u_int	fstamp;			/* NTP filestamp */
-char	*hostname = NULL;	/* host name (subject name) */
-char	*groupname = NULL;	/* trusted host name (issuer name) */
-char	filename[MAXFILENAME + 1]; /* file name */
+char	hostbuf[MAXHOSTNAME + 1];
+char	*hostname = NULL;	/* host, used in cert filenames */
+char	*groupname = NULL;	/* group name */
+char	certnamebuf[2 * sizeof(hostbuf)];
+char	*certname = NULL;	/* certificate subject/issuer name */
 char	*passwd1 = NULL;	/* input private key password */
 char	*passwd2 = NULL;	/* output private key password */
-#ifdef OPENSSL
+char	filename[MAXFILENAME + 1]; /* file name */
+#ifdef AUTOKEY
+u_int	modulus = PLEN;		/* prime modulus size (bits) */
+u_int	modulus2 = ILEN;	/* identity modulus size (bits) */
 long	d0, d1, d2, d3;		/* callback counters */
-#endif /* OPENSSL */
+const EVP_CIPHER * cipher = NULL;
+#endif	/* AUTOKEY */
 
 #ifdef SYS_WINNT
 BOOL init_randfile();
 
 /*
- * Don't try to follow symbolic links
+ * Don't try to follow symbolic links on Windows.  Assume link == file.
  */
 int
-readlink(char *link, char *file, int len)
+readlink(
+	char *	link,
+	char *	file,
+	int	len
+	)
 {
-	return (-1);
+	return (int)strlen(file); /* assume no overflow possible */
 }
 
 /*
- * Don't try to create a symbolic link for now.
- * Just move the file to the name you need.
+ * Don't try to create symbolic links on Windows, that is supported on
+ * Vista and later only.  Instead, if CreateHardLink is available (XP
+ * and later), hardlink the linkname to the original filename.  On
+ * earlier systems, user must rename file to match expected link for
+ * ntpd to find it.  To allow building a ntp-keygen.exe which loads on
+ * Windows pre-XP, runtime link to CreateHardLinkA().
  */
 int
-symlink(char *filename, char *linkname) {
-	DeleteFile(linkname);
-	MoveFile(filename, linkname);
-	return (0);
+symlink(
+	char *	filename,
+	char*	linkname
+	)
+{
+	typedef BOOL (WINAPI *PCREATEHARDLINKA)(
+		__in LPCSTR	lpFileName,
+		__in LPCSTR	lpExistingFileName,
+		__reserved LPSECURITY_ATTRIBUTES lpSA
+		);
+	static PCREATEHARDLINKA pCreateHardLinkA;
+	static int		tried;
+	HMODULE			hDll;
+	FARPROC			pfn;
+	int			link_created;
+	int			saved_errno;
+
+	if (!tried) {
+		tried = TRUE;
+		hDll = LoadLibrary("kernel32");
+		pfn = GetProcAddress(hDll, "CreateHardLinkA");
+		pCreateHardLinkA = (PCREATEHARDLINKA)pfn;
+	}
+
+	if (NULL == pCreateHardLinkA) {
+		errno = ENOSYS;
+		return -1;
+	}
+
+	link_created = (*pCreateHardLinkA)(linkname, filename, NULL);
+	
+	if (link_created)
+		return 0;
+
+	saved_errno = GetLastError();	/* yes we play loose */
+	mfprintf(stderr, "Create hard link %s to %s failed: %m\n",
+		 linkname, filename);
+	errno = saved_errno;
+	return -1;
 }
+
 void
 InitWin32Sockets() {
 	WORD wVersionRequested;
@@ -208,6 +253,33 @@ InitWin32Sockets() {
 }
 #endif /* SYS_WINNT */
 
+
+/*
+ * followlink() - replace filename with its target if symlink.
+ *
+ * Some readlink() implementations do not null-terminate the result.
+ */
+void
+followlink(
+	char *	fname,
+	size_t	bufsiz
+	)
+{
+	int len;
+
+	REQUIRE(bufsiz > 0);
+
+	len = readlink(fname, fname, (int)bufsiz);
+	if (len < 0 ) {
+		fname[0] = '\0';
+		return;
+	}
+	if (len > (int)bufsiz - 1)
+		len = (int)bufsiz - 1;
+	fname[len] = '\0';
+}
+
+
 /*
  * Main program
  */
@@ -219,7 +291,8 @@ main(
 {
 	struct timeval tv;	/* initialization vector */
 	int	md5key = 0;	/* generate MD5 keys */
-#ifdef OPENSSL
+	int	optct;		/* option count */
+#ifdef AUTOKEY
 	X509	*cert = NULL;	/* X509 certificate */
 	X509_EXTENSION *ext;	/* X509v3 extension */
 	EVP_PKEY *pkey_host = NULL; /* host key */
@@ -238,14 +311,17 @@ main(
 	const EVP_MD *ectx;	/* EVP digest */
 	char	pathbuf[MAXFILENAME + 1];
 	const char *scheme = NULL; /* digest/signature scheme */
-	char	*exten = NULL;	/* private extension */
+	const char *ciphername = NULL; /* to encrypt priv. key */
+	const char *exten = NULL;	/* private extension */
 	char	*grpkey = NULL;	/* identity extension */
 	int	nid;		/* X509 digest/signature scheme */
 	FILE	*fstr = NULL;	/* file handle */
-#define iffsw   HAVE_OPT(ID_KEY)
-#endif /* OPENSSL */
-	char	hostbuf[MAXHOSTNAME + 1];
 	char	groupbuf[MAXHOSTNAME + 1];
+	u_int	temp;
+	BIO *	bp;
+	int	i, cnt;
+	char *	ptr;
+#endif	/* AUTOKEY */
 
 	progname = argv[0];
 
@@ -259,23 +335,28 @@ main(
 
 #ifdef OPENSSL
 	ssl_check_version();
-#endif /* OPENSSL */
+#endif	/* OPENSSL */
+
+	ntp_crypto_srandom();
 
 	/*
 	 * Process options, initialize host name and timestamp.
+	 * gethostname() won't null-terminate if hostname is exactly the
+	 * length provided for the buffer.
 	 */
-	gethostname(hostbuf, MAXHOSTNAME);
+	gethostname(hostbuf, sizeof(hostbuf) - 1);
+	hostbuf[COUNTOF(hostbuf) - 1] = '\0';
 	hostname = hostbuf;
-	gettimeofday(&tv, 0);
-
+	groupname = hostbuf;
+	passwd1 = hostbuf;
+	passwd2 = NULL;
+	GETTIMEOFDAY(&tv, NULL);
 	epoch = tv.tv_sec;
+	fstamp = (u_int)(epoch + JAN_1970);
 
-	{
-		int optct = ntpOptionProcess(&ntp_keygenOptions,
-					     argc, argv);
-		argc -= optct;
-		argv += optct;
-	}
+	optct = ntpOptionProcess(&ntp_keygenOptions, argc, argv);
+	argc -= optct;	// Just in case we care later.
+	argv += optct;	// Just in case we care later.
 
 #ifdef OPENSSL
 	if (SSLeay() == SSLEAY_VERSION_NUMBER)
@@ -286,23 +367,22 @@ main(
 			OPENSSL_VERSION_TEXT, SSLeay_version(SSLEAY_VERSION));
 #endif /* OPENSSL */
 
-	debug = DESC(DEBUG_LEVEL).optOccCt;
+	debug = OPT_VALUE_SET_DEBUG_LEVEL;
+
 	if (HAVE_OPT( MD5KEY ))
 		md5key++;
+#ifdef AUTOKEY
+	if (HAVE_OPT( PASSWORD ))
+		passwd1 = estrdup(OPT_ARG( PASSWORD ));
 
-#ifdef OPENSSL
-	passwd1 = hostbuf;
-	if (HAVE_OPT( PVT_PASSWD ))
-		passwd1 = strdup(OPT_ARG( PVT_PASSWD ));
-
-	if (HAVE_OPT( GET_PVT_PASSWD ))
-		passwd2 = strdup(OPT_ARG( GET_PVT_PASSWD ));
+	if (HAVE_OPT( EXPORT_PASSWD ))
+		passwd2 = estrdup(OPT_ARG( EXPORT_PASSWD ));
 
 	if (HAVE_OPT( HOST_KEY ))
 		hostkey++;
 
 	if (HAVE_OPT( SIGN_KEY ))
-		sign = strdup(OPT_ARG( SIGN_KEY ));
+		sign = estrdup(OPT_ARG( SIGN_KEY ));
 
 	if (HAVE_OPT( GQ_PARAMS ))
 		gqkey++;
@@ -318,17 +398,27 @@ main(
 		mvpar++;
 		nkeys = OPT_VALUE_MV_KEYS;
 	}
+
+	if (HAVE_OPT( IMBITS ))
+		modulus2 = OPT_VALUE_IMBITS;
+
 	if (HAVE_OPT( MODULUS ))
 		modulus = OPT_VALUE_MODULUS;
 
 	if (HAVE_OPT( CERTIFICATE ))
 		scheme = OPT_ARG( CERTIFICATE );
 
-	if (HAVE_OPT( SUBJECT_NAME ))
-		hostname = strdup(OPT_ARG( SUBJECT_NAME ));
+	if (HAVE_OPT( CIPHER ))
+		ciphername = OPT_ARG( CIPHER );
 
-	if (HAVE_OPT( ISSUER_NAME ))
-		groupname = strdup(OPT_ARG( ISSUER_NAME ));
+	if (HAVE_OPT( SUBJECT_NAME ))
+		hostname = estrdup(OPT_ARG( SUBJECT_NAME ));
+
+	if (HAVE_OPT( IDENT ))
+		groupname = estrdup(OPT_ARG( IDENT ));
+
+	if (HAVE_OPT( LIFETIME ))
+		lifetime = OPT_VALUE_LIFETIME;
 
 	if (HAVE_OPT( PVT_CERT ))
 		exten = EXT_KEY_PRIVATE;
@@ -336,15 +426,44 @@ main(
 	if (HAVE_OPT( TRUSTED_CERT ))
 		exten = EXT_KEY_TRUST;
 
+	/*
+	 * Remove the group name from the hostname variable used
+	 * in host and sign certificate file names.
+	 */
+	if (hostname != hostbuf)
+		ptr = strchr(hostname, '@');
+	else
+		ptr = NULL;
+	if (ptr != NULL) {
+		*ptr = '\0';
+		groupname = estrdup(ptr + 1);
+		/* -s @group is equivalent to -i group, host unch. */
+		if (ptr == hostname)
+			hostname = hostbuf;
+	}
+
+	/*
+	 * Derive host certificate issuer/subject names from host name
+	 * and optional group.  If no groupname is provided, the issuer
+	 * and subject is the hostname with no '@group', and the
+	 * groupname variable is pointed to hostname for use in IFF, GQ,
+	 * and MV parameters file names.
+	 */
+	if (groupname == hostbuf) {
+		certname = hostname;
+	} else {
+		snprintf(certnamebuf, sizeof(certnamebuf), "%s@%s",
+			 hostname, groupname);
+		certname = certnamebuf;
+	}
+
 	/*
 	 * Seed random number generator and grow weeds.
 	 */
 	ERR_load_crypto_strings();
 	OpenSSL_add_all_algorithms();
 	if (!RAND_status()) {
-		u_int	temp;
-
-		if (RAND_file_name(pathbuf, MAXFILENAME) == NULL) {
+		if (RAND_file_name(pathbuf, sizeof(pathbuf)) == NULL) {
 			fprintf(stderr, "RAND_file_name %s\n",
 			    ERR_error_string(ERR_get_error(), NULL));
 			exit (-1);
@@ -360,11 +479,22 @@ main(
 		    "Random seed file %s %u bytes\n", pathbuf, temp);
 		RAND_add(&epoch, sizeof(epoch), 4.0);
 	}
+#endif	/* AUTOKEY */
 
+	/*
+	 * Create new unencrypted MD5 keys file if requested. If this
+	 * option is selected, ignore all other options.
+	 */
+	if (md5key) {
+		gen_md5("md5");
+		exit (0);
+	}
+
+#ifdef AUTOKEY
 	/*
 	 * Load previous certificate if available.
 	 */
-	sprintf(filename, "ntpkey_cert_%s", hostname);
+	snprintf(filename, sizeof(filename), "ntpkey_cert_%s", hostname);
 	if ((fstr = fopen(filename, "r")) != NULL) {
 		cert = PEM_read_X509(fstr, NULL, NULL, NULL);
 		fclose(fstr);
@@ -391,10 +521,6 @@ main(
 		 * whether this is a trusted or private certificate.
 		 */
 		if (exten == NULL) {
-			BIO	*bp;
-			int	i, cnt;
-			char	*ptr;
-
 			ptr = strstr(groupbuf, "CN=");
 			cnt = X509_get_ext_count(cert);
 			for (i = 0; i < cnt; i++) {
@@ -412,33 +538,23 @@ main(
 					else if (strcmp(pathbuf,
 					    "Private") == 0)
 						exten = EXT_KEY_PRIVATE;
-					if (groupname == NULL)
-						groupname = ptr + 3;
+					certname = estrdup(ptr + 3);
 				}
 			}
 		}
 	}
 	if (scheme == NULL)
 		scheme = "RSA-MD5";
-	if (groupname == NULL)
-		groupname = hostname;
+	if (ciphername == NULL)
+		ciphername = "des-ede3-cbc";
+	cipher = EVP_get_cipherbyname(ciphername);
+	if (cipher == NULL) {
+		fprintf(stderr, "Unknown cipher %s\n", ciphername);
+		exit(-1);
+	}
 	fprintf(stderr, "Using host %s group %s\n", hostname,
 	    groupname);
-	if ((iffkey || gqkey || mvkey) && exten == NULL)
-		fprintf(stderr,
-		    "Warning: identity files may not be useful with a nontrusted certificate.\n");
-#endif /* OPENSSL */
 
-	/*
-	 * Create new unencrypted MD5 keys file if requested. If this
-	 * option is selected, ignore all other options.
-	 */
-	if (md5key) {
-		gen_md5("md5");
-		exit (0);
-	}
-
-#ifdef OPENSSL
 	/*
 	 * Create a new encrypted RSA host key file if requested;
 	 * otherwise, look for an existing host key file. If not found,
@@ -448,10 +564,10 @@ main(
 	if (hostkey)
 		pkey_host = genkey("RSA", "host");
 	if (pkey_host == NULL) {
-		sprintf(filename, "ntpkey_host_%s", hostname);
+		snprintf(filename, sizeof(filename), "ntpkey_host_%s", hostname);
 		pkey_host = readkey(filename, passwd1, &fstamp, NULL);
 		if (pkey_host != NULL) {
-			readlink(filename, filename, sizeof(filename));
+			followlink(filename, sizeof(filename));
 			fprintf(stderr, "Using host key %s\n",
 			    filename);
 		} else {
@@ -460,7 +576,7 @@ main(
 	}
 	if (pkey_host == NULL) {
 		fprintf(stderr, "Generating host key fails\n");
-		exit (-1);
+		exit(-1);
 	}
 
 	/*
@@ -471,13 +587,14 @@ main(
 	if (sign != NULL)
 		pkey_sign = genkey(sign, "sign");
 	if (pkey_sign == NULL) {
-		sprintf(filename, "ntpkey_sign_%s", hostname);
+		snprintf(filename, sizeof(filename), "ntpkey_sign_%s",
+			 hostname);
 		pkey_sign = readkey(filename, passwd1, &fstamp, NULL);
 		if (pkey_sign != NULL) {
-			readlink(filename, filename, sizeof(filename));
+			followlink(filename, sizeof(filename));
 			fprintf(stderr, "Using sign key %s\n",
 			    filename);
-		} else if (pkey_host != NULL) {
+		} else {
 			pkey_sign = pkey_host;
 			fprintf(stderr, "Using host key as sign key\n");
 		}
@@ -491,10 +608,11 @@ main(
 	if (gqkey)
 		pkey_gqkey = gen_gqkey("gqkey");
 	if (pkey_gqkey == NULL) {
-		sprintf(filename, "ntpkey_gqkey_%s", groupname);
+		snprintf(filename, sizeof(filename), "ntpkey_gqkey_%s",
+		    groupname);
 		pkey_gqkey = readkey(filename, passwd1, &fstamp, NULL);
 		if (pkey_gqkey != NULL) {
-			readlink(filename, filename, sizeof(filename));
+			followlink(filename, sizeof(filename));
 			fprintf(stderr, "Using GQ parameters %s\n",
 			    filename);
 		}
@@ -510,9 +628,8 @@ main(
 	if (pkey_gqkey != NULL && HAVE_OPT(ID_KEY)) {
 		RSA	*rsa;
 
-		epoch = fstamp - JAN_1970;
-		sprintf(filename, "ntpkey_gqpar_%s.%u", groupname,
-		    fstamp);
+		snprintf(filename, sizeof(filename),
+		    "ntpkey_gqpar_%s.%u", groupname, fstamp);
 		fprintf(stderr, "Writing GQ parameters %s to stdout\n",
 		    filename);
 		fprintf(stdout, "# %s\n# %s\n", filename,
@@ -522,9 +639,9 @@ main(
 		BN_copy(rsa->q, BN_value_one());
 		pkey = EVP_PKEY_new();
 		EVP_PKEY_assign_RSA(pkey, rsa);
-		PEM_write_PrivateKey(stdout, pkey, NULL, NULL, 0, NULL,
-		    NULL);
-		fclose(stdout);
+		PEM_write_PKCS8PrivateKey(stdout, pkey, NULL, NULL, 0,
+		    NULL, NULL);
+		fflush(stdout);
 		if (debug)
 			RSA_print_fp(stderr, rsa, 0);
 	}
@@ -535,8 +652,8 @@ main(
 	if (pkey_gqkey != NULL && passwd2 != NULL) {
 		RSA	*rsa;
 
-		sprintf(filename, "ntpkey_gqkey_%s.%u", groupname,
-		    fstamp);
+		snprintf(filename, sizeof(filename),
+		    "ntpkey_gqkey_%s.%u", groupname, fstamp);
 		fprintf(stderr, "Writing GQ keys %s to stdout\n",
 		    filename);
 		fprintf(stdout, "# %s\n# %s\n", filename,
@@ -544,9 +661,9 @@ main(
 		rsa = pkey_gqkey->pkey.rsa;
 		pkey = EVP_PKEY_new();
 		EVP_PKEY_assign_RSA(pkey, rsa);
-		PEM_write_PrivateKey(stdout, pkey,
-		    EVP_des_cbc(), NULL, 0, NULL, passwd2);
-		fclose(stdout);
+		PEM_write_PKCS8PrivateKey(stdout, pkey, cipher, NULL, 0,
+		    NULL, passwd2);
+		fflush(stdout);
 		if (debug)
 			RSA_print_fp(stderr, rsa, 0);
 	}
@@ -558,10 +675,11 @@ main(
 	if (iffkey)
 		pkey_iffkey = gen_iffkey("iffkey");
 	if (pkey_iffkey == NULL) {
-		sprintf(filename, "ntpkey_iffkey_%s", groupname);
+		snprintf(filename, sizeof(filename), "ntpkey_iffkey_%s",
+		    groupname);
 		pkey_iffkey = readkey(filename, passwd1, &fstamp, NULL);
 		if (pkey_iffkey != NULL) {
-			readlink(filename, filename, sizeof(filename));
+			followlink(filename, sizeof(filename));
 			fprintf(stderr, "Using IFF keys %s\n",
 			    filename);
 		}
@@ -575,9 +693,8 @@ main(
 	if (pkey_iffkey != NULL && HAVE_OPT(ID_KEY)) {
 		DSA	*dsa;
 
-		epoch = fstamp - JAN_1970;
-		sprintf(filename, "ntpkey_iffpar_%s.%u", groupname,
-		    fstamp);
+		snprintf(filename, sizeof(filename),
+		    "ntpkey_iffpar_%s.%u", groupname, fstamp);
 		fprintf(stderr, "Writing IFF parameters %s to stdout\n",
 		    filename);
 		fprintf(stdout, "# %s\n# %s\n", filename,
@@ -586,9 +703,9 @@ main(
 		BN_copy(dsa->priv_key, BN_value_one());
 		pkey = EVP_PKEY_new();
 		EVP_PKEY_assign_DSA(pkey, dsa);
-		PEM_write_PrivateKey(stdout, pkey, NULL, NULL, 0, NULL,
-		    NULL);
-		fclose(stdout);
+		PEM_write_PKCS8PrivateKey(stdout, pkey, NULL, NULL, 0,
+		    NULL, NULL);
+		fflush(stdout);
 		if (debug)
 			DSA_print_fp(stderr, dsa, 0);
 	}
@@ -599,9 +716,8 @@ main(
 	if (pkey_iffkey != NULL && passwd2 != NULL) {
 		DSA	*dsa;
 
-		epoch = fstamp - JAN_1970;
-		sprintf(filename, "ntpkey_iffkey_%s.%u", groupname,
-		    fstamp);
+		snprintf(filename, sizeof(filename),
+		    "ntpkey_iffkey_%s.%u", groupname, fstamp);
 		fprintf(stderr, "Writing IFF keys %s to stdout\n",
 		    filename);
 		fprintf(stdout, "# %s\n# %s\n", filename,
@@ -609,9 +725,9 @@ main(
 		dsa = pkey_iffkey->pkey.dsa;
 		pkey = EVP_PKEY_new();
 		EVP_PKEY_assign_DSA(pkey, dsa);
-		PEM_write_PrivateKey(stdout, pkey, EVP_des_cbc(), NULL,
-		    0, NULL, passwd2);
-		fclose(stdout);
+		PEM_write_PKCS8PrivateKey(stdout, pkey, cipher, NULL, 0,
+		    NULL, passwd2);
+		fflush(stdout);
 		if (debug)
 			DSA_print_fp(stderr, dsa, 0);
 	}
@@ -623,11 +739,12 @@ main(
 	if (mvkey)
 		pkey_mvkey = gen_mvkey("mv", pkey_mvpar);
 	if (pkey_mvkey == NULL) {
-		sprintf(filename, "ntpkey_mvta_%s", groupname);
+		snprintf(filename, sizeof(filename), "ntpkey_mvta_%s",
+		    groupname);
 		pkey_mvkey = readkey(filename, passwd1, &fstamp,
-		   pkey_mvpar);
+		    pkey_mvpar);
 		if (pkey_mvkey != NULL) {
-			readlink(filename, filename, sizeof(filename));
+			followlink(filename, sizeof(filename));
 			fprintf(stderr, "Using MV keys %s\n",
 			    filename);
 		}
@@ -639,17 +756,16 @@ main(
 	 * associated with client key 1.
 	 */
 	if (pkey_mvkey != NULL && HAVE_OPT(ID_KEY)) {
-		epoch = fstamp - JAN_1970;
-		sprintf(filename, "ntpkey_mvpar_%s.%u", groupname,
-		    fstamp);
+		snprintf(filename, sizeof(filename),
+		    "ntpkey_mvpar_%s.%u", groupname, fstamp);
 		fprintf(stderr, "Writing MV parameters %s to stdout\n",
 		    filename);
 		fprintf(stdout, "# %s\n# %s\n", filename,
 		    ctime(&epoch));
 		pkey = pkey_mvpar[2];
-		PEM_write_PrivateKey(stdout, pkey, NULL, NULL, 0, NULL,
-		    NULL);
-		fclose(stdout);
+		PEM_write_PKCS8PrivateKey(stdout, pkey, NULL, NULL, 0,
+		    NULL, NULL);
+		fflush(stdout);
 		if (debug)
 			DSA_print_fp(stderr, pkey->pkey.dsa, 0);
 	}
@@ -658,32 +774,23 @@ main(
 	 * Write the encrypted MV server keys to the stdout stream.
 	 */
 	if (pkey_mvkey != NULL && passwd2 != NULL) {
-		epoch = fstamp - JAN_1970;
-		sprintf(filename, "ntpkey_mvkey_%s.%u", groupname,
-		    fstamp);
+		snprintf(filename, sizeof(filename),
+		    "ntpkey_mvkey_%s.%u", groupname, fstamp);
 		fprintf(stderr, "Writing MV keys %s to stdout\n",
 		    filename);
 		fprintf(stdout, "# %s\n# %s\n", filename,
 		    ctime(&epoch));
 		pkey = pkey_mvpar[1];
-		PEM_write_PrivateKey(stdout, pkey, EVP_des_cbc(), NULL,
-		    0, NULL, passwd2);
-		fclose(stdout);
+		PEM_write_PKCS8PrivateKey(stdout, pkey, cipher, NULL, 0,
+		    NULL, passwd2);
+		fflush(stdout);
 		if (debug)
 			DSA_print_fp(stderr, pkey->pkey.dsa, 0);
 	}
 
 	/*
-	 * Don't generate a certificate if no host keys or extracting
-	 * encrypted or nonencrypted keys to the standard output stream.
-	 */
-	if (pkey_host == NULL || HAVE_OPT(ID_KEY) || passwd2 != NULL)
-		exit (0);
-
-	/*
-	 * Decode the digest/signature scheme. If trusted, set the
-	 * subject and issuer names to the group name; if not set both
-	 * to the host name.
+	 * Decode the digest/signature scheme and create the
+	 * certificate. Do this every time we run the program.
 	 */
 	ectx = EVP_get_digestbyname(scheme);
 	if (ectx == NULL) {
@@ -692,12 +799,9 @@ main(
 		    scheme);
 			exit (-1);
 	}
-	if (exten == NULL)
-		x509(pkey_sign, ectx, grpkey, exten, hostname);
-	else
-		x509(pkey_sign, ectx, grpkey, exten, groupname);
-#endif /* OPENSSL */
-	exit (0);
+	x509(pkey_sign, ectx, grpkey, exten, certname);
+#endif	/* AUTOKEY */
+	exit(0);
 }
 
 
@@ -708,7 +812,7 @@ main(
  */
 int
 gen_md5(
-	char	*id		/* file name id */
+	const char *id		/* file name id */
 	)
 {
 	u_char	md5key[MD5SIZE + 1];	/* MD5 key */
@@ -718,23 +822,29 @@ gen_md5(
 	u_char	keystr[MD5SIZE];
 	u_char	hexstr[2 * MD5SIZE + 1];
 	u_char	hex[] = "0123456789abcdef";
-#endif /* OPENSSL */
+#endif	/* OPENSSL */
 
 	str = fheader("MD5key", id, groupname);
-	ntp_srandom((u_long)epoch);
 	for (i = 1; i <= MD5KEYS; i++) {
 		for (j = 0; j < MD5SIZE; j++) {
-			int temp;
+			u_char temp;
 
 			while (1) {
-				temp = ntp_random() & 0xff;
+				int rc;
+
+				rc = ntp_crypto_random_buf(
+				    &temp, sizeof(temp));
+				if (-1 == rc) {
+					fprintf(stderr, "ntp_crypto_random_buf() failed.\n");
+					exit (-1);
+				}
 				if (temp == '#')
 					continue;
 
 				if (temp > 0x20 && temp < 0x7f)
 					break;
 			}
-			md5key[j] = (u_char)temp;
+			md5key[j] = temp;
 		}
 		md5key[j] = '\0';
 		fprintf(str, "%2d MD5 %s  # MD5 key\n", i,
@@ -751,13 +861,13 @@ gen_md5(
 		fprintf(str, "%2d SHA1 %s  # SHA1 key\n", i + MD5KEYS,
 		    hexstr);
 	}
-#endif /* OPENSSL */
+#endif	/* OPENSSL */
 	fclose(str);
 	return (1);
 }
 
 
-#ifdef OPENSSL
+#ifdef AUTOKEY
 /*
  * readkey - load cryptographic parameters and keys
  *
@@ -849,7 +959,7 @@ readkey(
  */
 EVP_PKEY *			/* public/private key pair */
 gen_rsa(
-	char	*id		/* file name id */
+	const char *id		/* file name id */
 	)
 {
 	EVP_PKEY *pkey;		/* private key */
@@ -857,7 +967,7 @@ gen_rsa(
 	FILE	*str;
 
 	fprintf(stderr, "Generating RSA keys (%d bits)...\n", modulus);
-	rsa = RSA_generate_key(modulus, 3, cb, "RSA");
+	rsa = RSA_generate_key(modulus, 65537, cb, _UC("RSA"));
 	fprintf(stderr, "\n");
 	if (rsa == NULL) {
 		fprintf(stderr, "RSA generate keys fails\n%s\n",
@@ -888,7 +998,7 @@ gen_rsa(
 		str = fheader("RSAhost", id, hostname);
 	pkey = EVP_PKEY_new();
 	EVP_PKEY_assign_RSA(pkey, rsa);
-	PEM_write_PrivateKey(str, pkey, EVP_des_cbc(), NULL, 0, NULL,
+	PEM_write_PKCS8PrivateKey(str, pkey, cipher, NULL, 0, NULL,
 	    passwd1);
 	fclose(str);
 	if (debug)
@@ -902,7 +1012,7 @@ gen_rsa(
  */
 EVP_PKEY *			/* public/private key pair */
 gen_dsa(
-	char	*id		/* file name id */
+	const char *id		/* file name id */
 	)
 {
 	EVP_PKEY *pkey;		/* private key */
@@ -917,7 +1027,7 @@ gen_dsa(
 	    "Generating DSA parameters (%d bits)...\n", modulus);
 	RAND_bytes(seed, sizeof(seed));
 	dsa = DSA_generate_parameters(modulus, seed, sizeof(seed), NULL,
-	    NULL, cb, "DSA");
+	    NULL, cb, _UC("DSA"));
 	fprintf(stderr, "\n");
 	if (dsa == NULL) {
 		fprintf(stderr, "DSA generate parameters fails\n%s\n",
@@ -943,7 +1053,7 @@ gen_dsa(
 	str = fheader("DSAsign", id, hostname);
 	pkey = EVP_PKEY_new();
 	EVP_PKEY_assign_DSA(pkey, dsa);
-	PEM_write_PrivateKey(str, pkey, EVP_des_cbc(), NULL, 0, NULL,
+	PEM_write_PKCS8PrivateKey(str, pkey, cipher, NULL, 0, NULL,
 	    passwd1);
 	fclose(str);
 	if (debug)
@@ -1004,7 +1114,7 @@ gen_dsa(
  */
 EVP_PKEY *			/* DSA cuckoo nest */
 gen_iffkey(
-	char	*id		/* file name id */
+	const char *id		/* file name id */
 	)
 {
 	EVP_PKEY *pkey;		/* private key */
@@ -1022,7 +1132,7 @@ gen_iffkey(
 	    modulus2);
 	RAND_bytes(seed, sizeof(seed));
 	dsa = DSA_generate_parameters(modulus2, seed, sizeof(seed), NULL,
-	    NULL, cb, "IFF");
+	    NULL, cb, _UC("IFF"));
 	fprintf(stderr, "\n");
 	if (dsa == NULL) {
 		fprintf(stderr, "DSA generate parameters fails\n%s\n",
@@ -1110,7 +1220,7 @@ gen_iffkey(
 	str = fheader("IFFkey", id, groupname);
 	pkey = EVP_PKEY_new();
 	EVP_PKEY_assign_DSA(pkey, dsa);
-	PEM_write_PrivateKey(str, pkey, EVP_des_cbc(), NULL, 0, NULL,
+	PEM_write_PKCS8PrivateKey(str, pkey, cipher, NULL, 0, NULL,
 	    passwd1);
 	fclose(str);
 	if (debug)
@@ -1182,7 +1292,7 @@ gen_iffkey(
  */
 EVP_PKEY *			/* RSA cuckoo nest */
 gen_gqkey(
-	char	*id		/* file name id */
+	const char *id		/* file name id */
 	)
 {
 	EVP_PKEY *pkey;		/* private key */
@@ -1198,15 +1308,15 @@ gen_gqkey(
 	fprintf(stderr,
 	    "Generating GQ parameters (%d bits)...\n",
 	     modulus2);
-	rsa = RSA_generate_key(modulus2, 3, cb, "GQ");
+	rsa = RSA_generate_key(modulus2, 65537, cb, _UC("GQ"));
 	fprintf(stderr, "\n");
 	if (rsa == NULL) {
 		fprintf(stderr, "RSA generate keys fails\n%s\n",
 		    ERR_error_string(ERR_get_error(), NULL));
 		return (NULL);
 	}
-	ctx = BN_CTX_new(); u = BN_new(); v = BN_new();
-	g = BN_new(); k = BN_new(); r = BN_new(); y = BN_new();
+	u = BN_new(); v = BN_new(); g = BN_new();
+	k = BN_new(); r = BN_new(); y = BN_new();
 
 	/*
 	 * Generate the group key b, which is saved in the e member of
@@ -1307,7 +1417,7 @@ gen_gqkey(
 	str = fheader("GQkey", id, groupname);
 	pkey = EVP_PKEY_new();
 	EVP_PKEY_assign_RSA(pkey, rsa);
-	PEM_write_PrivateKey(str, pkey, EVP_des_cbc(), NULL, 0, NULL,
+	PEM_write_PKCS8PrivateKey(str, pkey, cipher, NULL, 0, NULL,
 	    passwd1);
 	fclose(str);
 	if (debug)
@@ -1376,8 +1486,8 @@ gen_gqkey(
  *
  * How it works
  *
- * The scheme goes like this. Bob has the server values (p, E, q, gbar,
- * ghat) and Alice has the client values (p, xbar, xhat).
+ * The scheme goes like this. Bob has the server values (p, E, q,
+ * gbar, ghat) and Alice has the client values (p, xbar, xhat).
  *
  * Alice rolls new random nonce r mod p and sends to Bob in the MV
  * request message. Bob rolls random nonce k mod q, encrypts y = r E^k
@@ -1391,7 +1501,7 @@ gen_gqkey(
  */
 EVP_PKEY *			/* DSA cuckoo nest */
 gen_mvkey(
-	char	*id,		/* file name id */
+	const char *id,		/* file name id */
 	EVP_PKEY **evpars	/* parameter list pointer */
 	)
 {
@@ -1540,7 +1650,6 @@ gen_mvkey(
 	 */
 	for (i = 0; i <= n; i++) {
 		a[i] = BN_new();
-
 		BN_one(a[i]);
 	}
 	for (j = 1; j <= n; j++) {
@@ -1560,7 +1669,6 @@ gen_mvkey(
 	 */
 	for (i = 0; i <= n; i++) {
 		g[i] = BN_new();
-
 		BN_mod_exp(g[i], dsa->g, a[i], dsa->p, ctx);
 	}
 
@@ -1639,6 +1747,7 @@ gen_mvkey(
 		for (i = 1; i <= n; i++) {
 			if (i == j)
 				continue;
+
 			BN_mod_exp(u, x[i], v, dsa->q, ctx);
 			BN_add(xbar[j], xbar[j], u);
 		}
@@ -1655,7 +1764,6 @@ gen_mvkey(
 	 * additional keys, so we sail on with only token revocations.
 	 */
 	s = BN_new();
-
 	BN_copy(s, dsa->q);
 	BN_div(s, u, s, s1[n], ctx);
 
@@ -1668,7 +1776,6 @@ gen_mvkey(
 	 * changed.
 	 */
 	bige = BN_new(); gbar = BN_new(); ghat = BN_new();
-
 	BN_mod_exp(bige, biga, s, dsa->p, ctx);
 	BN_mod_exp(gbar, dsa->g, s, dsa->p, ctx);
 	BN_mod_mul(v, s, b, dsa->q, ctx);
@@ -1712,7 +1819,7 @@ gen_mvkey(
 	BN_copy(dsa->pub_key, b);
 	pkey = EVP_PKEY_new();
 	EVP_PKEY_assign_DSA(pkey, dsa);
-	PEM_write_PrivateKey(str, pkey, EVP_des_cbc(), NULL, 0, NULL,
+	PEM_write_PKCS8PrivateKey(str, pkey, cipher, NULL, 0, NULL,
 	    passwd1);
 	evpars[i++] = pkey;
 	if (debug)
@@ -1738,7 +1845,7 @@ gen_mvkey(
 	dsa2->pub_key = BN_dup(ghat);
 	pkey1 = EVP_PKEY_new();
 	EVP_PKEY_assign_DSA(pkey1, dsa2);
-	PEM_write_PrivateKey(str, pkey1, EVP_des_cbc(), NULL, 0, NULL,
+	PEM_write_PKCS8PrivateKey(str, pkey1, cipher, NULL, 0, NULL,
 	    passwd1);
 	evpars[i++] = pkey1;
 	if (debug)
@@ -1756,7 +1863,6 @@ gen_mvkey(
 	fprintf(stderr, "Generating %d MV client keys\n", n);
 	for (j = 1; j <= n; j++) {
 		sdsa = DSA_new();
-
 		sdsa->p = BN_dup(dsa->p);
 		sdsa->q = BN_dup(BN_value_one()); 
 		sdsa->g = BN_dup(BN_value_one()); 
@@ -1764,7 +1870,7 @@ gen_mvkey(
 		sdsa->pub_key = BN_dup(xhat[j]);
 		pkey1 = EVP_PKEY_new();
 		EVP_PKEY_set1_DSA(pkey1, sdsa);
-		PEM_write_PrivateKey(str, pkey1, EVP_des_cbc(), NULL, 0,
+		PEM_write_PKCS8PrivateKey(str, pkey1, cipher, NULL, 0,
 		    NULL, passwd1);
 		evpars[i++] = pkey1;
 		if (debug)
@@ -1817,11 +1923,11 @@ gen_mvkey(
  */
 int
 x509	(
-	EVP_PKEY *pkey,		/* generic signature algorithm */
-	const EVP_MD *md,	/* generic digest algorithm */
+	EVP_PKEY *pkey,		/* signing key */
+	const EVP_MD *md,	/* signature/digest scheme */
 	char	*gqpub,		/* identity extension (hex string) */
-	char	*exten,		/* private cert extension */
-	char	*name		/* subject/issuer namd */
+	const char *exten,	/* private cert extension */
+	char	*name		/* subject/issuer name */
 	)
 {
 	X509	*cert;		/* X509 certificate */
@@ -1848,15 +1954,15 @@ x509	(
 	X509_set_serialNumber(cert, serial);
 	ASN1_INTEGER_free(serial);
 	X509_time_adj(X509_get_notBefore(cert), 0L, &epoch);
-	X509_time_adj(X509_get_notAfter(cert), YEAR, &epoch);
+	X509_time_adj(X509_get_notAfter(cert), lifetime * SECSPERDAY, &epoch);
 	subj = X509_get_subject_name(cert);
 	X509_NAME_add_entry_by_txt(subj, "commonName", MBSTRING_ASC,
-	    (unsigned char *) name, strlen(name), -1, 0);
+	    (u_char *)name, -1, -1, 0);
 	subj = X509_get_issuer_name(cert);
 	X509_NAME_add_entry_by_txt(subj, "commonName", MBSTRING_ASC,
-	    (unsigned char *) name, strlen(name), -1, 0);
+	    (u_char *)name, -1, -1, 0);
 	if (!X509_set_pubkey(cert, pkey)) {
-		fprintf(stderr, "Assign key fails\n%s\n",
+		fprintf(stderr, "Assign certificate signing key fails\n%s\n",
 		    ERR_error_string(ERR_get_error(), NULL));
 		X509_free(cert);
 		return (0);
@@ -1874,7 +1980,7 @@ x509	(
 	fprintf(stderr, "%s: %s\n", LN_basic_constraints,
 	    BASIC_CONSTRAINTS);
 	ex = X509V3_EXT_conf_nid(NULL, NULL, NID_basic_constraints,
-	    BASIC_CONSTRAINTS);
+	    _UC(BASIC_CONSTRAINTS));
 	if (!X509_add_ext(cert, ex, -1)) {
 		fprintf(stderr, "Add extension field fails\n%s\n",
 		    ERR_error_string(ERR_get_error(), NULL));
@@ -1887,7 +1993,7 @@ x509	(
 	 * be used for.
 	 */
 	fprintf(stderr, "%s: %s\n", LN_key_usage, KEY_USAGE);
-	ex = X509V3_EXT_conf_nid(NULL, NULL, NID_key_usage, KEY_USAGE);
+	ex = X509V3_EXT_conf_nid(NULL, NULL, NID_key_usage, _UC(KEY_USAGE));
 	if (!X509_add_ext(cert, ex, -1)) {
 		fprintf(stderr, "Add extension field fails\n%s\n",
 		    ERR_error_string(ERR_get_error(), NULL));
@@ -1922,7 +2028,7 @@ x509	(
 	if (exten != NULL) {
 		fprintf(stderr, "%s: %s\n", LN_ext_key_usage, exten);
 		ex = X509V3_EXT_conf_nid(NULL, NULL,
-		    NID_ext_key_usage, exten);
+		    NID_ext_key_usage, _UC(exten));
 		if (!X509_add_ext(cert, ex, -1)) {
 			fprintf(stderr,
 			    "Add extension field fails\n%s\n",
@@ -1946,7 +2052,7 @@ x509	(
 	/*
 	 * Write the certificate encoded in PEM.
 	 */
-	sprintf(pathbuf, "%scert", id);
+	snprintf(pathbuf, sizeof(pathbuf), "%scert", id);
 	str = fheader(pathbuf, "cert", hostname);
 	PEM_write_X509(str, cert);
 	fclose(str);
@@ -2033,8 +2139,8 @@ cb	(
  */
 EVP_PKEY *			/* public/private key pair */
 genkey(
-	char	*type,		/* key type (RSA or DSA) */
-	char	*id		/* file name id */
+	const char *type,	/* key type (RSA or DSA) */
+	const char *id		/* file name id */
 	)
 {
 	if (type == NULL)
@@ -2048,7 +2154,7 @@ genkey(
 	fprintf(stderr, "Invalid %s key type %s\n", id, type);
 	return (NULL);
 }
-#endif /* OPENSSL */
+#endif	/* AUTOKEY */
 
 
 /*
@@ -2064,15 +2170,30 @@ fheader	(
 	FILE	*str;		/* file handle */
 	char	linkname[MAXFILENAME]; /* link name */
 	int	temp;
-
-	sprintf(filename, "ntpkey_%s_%s.%lu", file, owner, epoch +
-	    JAN_1970);
-	if ((str = fopen(filename, "w")) == NULL) {
+#ifdef HAVE_UMASK
+        mode_t  orig_umask;
+#endif
+        
+	snprintf(filename, sizeof(filename), "ntpkey_%s_%s.%u", file,
+	    owner, fstamp); 
+#ifdef HAVE_UMASK
+        orig_umask = umask( S_IWGRP | S_IRWXO );
+        str = fopen(filename, "w");
+        (void) umask(orig_umask);
+#else
+        str = fopen(filename, "w");
+#endif
+	if (str == NULL) {
 		perror("Write");
 		exit (-1);
 	}
-	sprintf(linkname, "ntpkey_%s_%s", ulink, owner);
-	remove(linkname);
+        if (strcmp(ulink, "md5") == 0) {
+          strcpy(linkname,"ntp.keys");
+        } else {
+          snprintf(linkname, sizeof(linkname), "ntpkey_%s_%s", ulink,
+                   hostname);
+        }
+	(void)remove(linkname);		/* The symlink() line below matters */
 	temp = symlink(filename, linkname);
 	if (temp < 0)
 		perror(file);
